/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): Romain Rouvoy______________________________________.
 
 ---------------------------------------------------------------------
 $Id: LifeCycleComponentIconProvider.java,v 1.1 2004/04/14 09:43:14 rouvoy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.icon;

import java.util.HashMap;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.SuperController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.browser.api.IconProvider;

/**
 * Provides a specific icon depending on the status of the component.
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.2
 */
public class LifeCycleComponentIconProvider 
  implements IconProvider
{
    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    /** The different icons */
    protected Map icons;
    
    /**
     * @return Returns the icons.
     */
    protected Map getIcons() {
        return icons;
    }
    /**
     * @param icons The icons to set.
     */
    protected void setIcons(Map icons) {
        this.icons = icons;
    }
    
    //==================================================================
    //
    // Constructors.
    // 
    //==================================================================
    
    /**
     * Load a icon from the classpath.
     * @param path the path for finding the requested icon.
     * @return the instance of icon found (null if not found).
     */
    protected Icon load(String path) {
        Icon icon = new ImageIcon(Thread.currentThread().getContextClassLoader().getResource(path));
        if (icon == null)
            System.err.println(path+" icon not found!");
        return icon ;
    }
    
    /**
     * Load a couple of icons based on the Fc+name+(Started|Stopped) pattern.
     * @param name the pattern to retrieve.
     */
    protected void loadFc(String name) {
        getIcons().put(name, 
                       new Icon[] {load("Fc"+name+"Started.png"),
                                   load("Fc"+name+"Stopped.png")});
    }
    
    /** Empty constructor. */
    public LifeCycleComponentIconProvider() {
        setIcons(new HashMap());
        loadFc("Primitive");
        loadFc("Composite");
        loadFc("Shared");
    }
    
    //==================================================================
    //
    // No internal method.
    //
    //==================================================================
    
    /**
     * Provides the couple of icons for a type of component (Primitive, Composite or Shared).
     * @param cpt the component to introspect.
     * @return the couple of icons associated to the component.
     */
    protected Icon[] loadIcon(Component cpt) {
        try {
            SuperController sc = Fractal.getSuperController(cpt);
            if (sc.getFcSuperComponents().length > 1)
                return (Icon[]) getIcons().get("Shared"); 
        } catch (NoSuchInterfaceException e) {}
        try {
            Fractal.getContentController(cpt);
            return (Icon[]) getIcons().get("Composite");
        } catch (NoSuchInterfaceException e) {}
        return (Icon[]) getIcons().get("Primitive"); 
    }
    
    //==================================================================
    //
    // Public methods for IconProvider interface.
    //
    //==================================================================
    
    /**
     * Provides the appropriate icon.
     * @param object the component to represent.
     * @return the associated icon.
     */
    public Icon newIcon(Object object) {
        Component ci = (Component)object;
        Icon[] icons = loadIcon(ci);
        if (icons == null) return null;
        LifeCycleController lcc=null;
        try {
            lcc = Fractal.getLifeCycleController(ci);
        } catch (NoSuchInterfaceException e) {
            return icons[0];
        }
        String status = lcc.getFcState();
        if (status.equals(LifeCycleController.STARTED))
            return icons[0];
        else if (status.equals(LifeCycleController.STOPPED))
            return icons[1];
        else
            return null;
    }
}