/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.reflect;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.common.ClassesInheritance;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;

/**
 * This class allows to browse a <code>java.lang.Class</code> object.
 * It displays<br>
 * <ul>
 *   <li>its fields</li>
 *   <li>its methods</li>
 *   <li>Its inner classes</li>
 * </ul>
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class ClassContext 
    implements Wrapper, Context {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The class to browse */
    protected Class class_;
    
    /** The comparator to use to sort the arrays */
    protected MemberComparator comparator_;
    
    //==================================================================
    //
    // Constructors.
    // 
    //==================================================================

    public ClassContext(){
        comparator_ = new MemberComparator();
    }

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    protected void addFields(Field[] fields, List entryList, ClassVisibilityConfig config){
        Arrays.sort(fields,comparator_);
        boolean displayType = config.get(ClassVisibilityConfig.ATTRIBUTE + "." + ClassVisibilityConfig.TYPE);
        boolean displayDeclaringClass = config.get(ClassVisibilityConfig.ATTRIBUTE + "." + ClassVisibilityConfig.DECLARING_CLASS) && config.get(ClassVisibilityConfig.FLAT_VIEW);  
        for (int i = 0; i < fields.length; i++) {
            if(ClassVisibility.hasToBeAdded(fields[i].getModifiers(), config, ClassVisibilityConfig.ATTRIBUTE)){
                StringBuffer name = new StringBuffer(fields[i].getName());
                if(displayType)
                    name.append(": " + ClassVisibility.getTypeName(fields[i].getType()));
                if(displayDeclaringClass)
                    name.append(" (from " + ClassVisibility.getTypeName(fields[i].getDeclaringClass()) + ")"); 
                entryList.add(new DefaultEntry(fields[i],new DefaultName(name.toString()),this));
            }
        }
    }

    protected void addMethods(Method[] methods, List entryList, ClassVisibilityConfig config){
        Arrays.sort(methods,comparator_);
        boolean displayParams = config.get(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.PARAMS);
        boolean displayReturnType = config.get(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.RETURN_TYPE);
        boolean displayDeclaringClass = config.get(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.DECLARING_CLASS) && config.get(ClassVisibilityConfig.FLAT_VIEW);
        for (int i = 0; i < methods.length; i++) {
            if(ClassVisibility.hasToBeAdded(methods[i].getModifiers(), config, ClassVisibilityConfig.METHOD)){
                StringBuffer name = new StringBuffer(methods[i].getName() + "(");
                if(displayParams){
                    Class[] params = methods[i].getParameterTypes();
                    for(int j=0 ; j<params.length ; j++) {
                        name.append(ClassVisibility.getTypeName(params[j]));
                        if(j<params.length-1)
                            name.append(", ");
                    }    
                }
                name.append(")");
                if(displayReturnType)
                    name.append(": " + ClassVisibility.getTypeName(methods[i].getReturnType()));
                if(displayDeclaringClass)
                    name.append(" (from " + ClassVisibility.getTypeName(methods[i].getDeclaringClass()) + ")");
                entryList.add(new DefaultEntry(methods[i],new DefaultName(name.toString()),this));
            }
        }
    }

    protected void addInnerClass(Class[] classes, List entryList, ClassVisibilityConfig config){
        Arrays.sort(classes,comparator_);
        boolean displayDeclaringClass = config.get(ClassVisibilityConfig.INNER_CLASS + "." + ClassVisibilityConfig.DECLARING_CLASS) && config.get(ClassVisibilityConfig.FLAT_VIEW);
        for (int i = 0; i < classes.length; i++) {
            if(ClassVisibility.hasToBeAdded(classes[i].getModifiers(), config, ClassVisibilityConfig.INNER_CLASS)){
                StringBuffer name = new StringBuffer(classes[i].getName().substring(classes[i].getName().lastIndexOf('$') + 1));
                if(displayDeclaringClass)
                    name.append(" (from " + ClassVisibility.getTypeName(classes[i].getDeclaringClass()) + ")");
                entryList.add(new DefaultEntry(classes[i],new DefaultName(name.toString()),this));
            }
        }
    }

    //==================================================================
    //
    // Public methods for Context interface.
    //
    //==================================================================

    /**
     * Returns an array containing the entries contained by the context.
     * @return The entries of the context.
     */
    public Entry[] getEntries() {
        
        ClassVisibilityConfig config = (ClassVisibilityConfig)ClassVisibility.config_.get(class_.getName());
        if(config == null)
            config = (ClassVisibilityConfig)ClassVisibility.config_.get(ClassVisibility.DEFAULT_CONFIG);    
            
        List entryList = new Vector();
        if (config.get(ClassVisibilityConfig.TREE_INHERITANCE)) {
            if(!config.get(ClassVisibilityConfig.FLAT_VIEW)){
                entryList.add(new DefaultEntry(new TreeInheritance(class_),new DefaultName("Tree inheritance"),this));
                addFields(class_.getDeclaredFields(), entryList, config);
                addMethods(class_.getDeclaredMethods(), entryList, config);
                addInnerClass(class_.getDeclaredClasses(), entryList, config);
            } else {
                ClassesInheritance classesInheritance = new ClassesInheritance(class_);
                List classesList = classesInheritance.getInheritClasses();
                List fields = new Vector();
                List methods = new Vector();
                List innerClasses = new Vector();
                Iterator it = classesList.iterator();
                while (it.hasNext()) {
					Class element = (Class) it.next();
                    // It is not necessary displaying interfaces from interface with flat view
                    if(!element.isInterface()){
                        fields.addAll(Arrays.asList(element.getDeclaredFields()));
                        methods.addAll(Arrays.asList(element.getDeclaredMethods()));
                        innerClasses.addAll(Arrays.asList(element.getDeclaredClasses()));
                    }
				}                 
                addFields((Field[])fields.toArray(new Field[0]), entryList, config);
                addMethods((Method[])methods.toArray(new Method[0]), entryList, config);
                addInnerClass((Class[])innerClasses.toArray(new Class[0]), entryList, config);
            }
        } else {
            addFields(class_.getDeclaredFields(), entryList, config);
            addMethods(class_.getDeclaredMethods(), entryList, config);
            addInnerClass(class_.getDeclaredClasses(), entryList, config);
        }

        return (Entry[])entryList.toArray(new Entry[0]);        
    }

    //==================================================================
    //
    // Public methods for Wrapper interface.
    //
    //==================================================================

    /**
     * Sets the wrapped object.
     * 
     * @param object The object to wrap.
     */
    public void 
    setWrapped(Object object) {
        class_ = (Class)object;
    }

    /**
     * Gets the wrapped object.
     * 
     * @return The wrapped object.
     */
    public Object 
    getWrapped() {
        return class_;
    }

}