/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.common;

import javax.swing.JPopupMenu;
import javax.swing.Icon;
import java.util.Vector;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.DropAction;
import org.objectweb.util.browser.api.Panel;
import org.objectweb.util.browser.api.Info;
import org.objectweb.util.browser.core.wrapper.AssociatedContextImpl;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.popup.AssociatedMenuImpl;
import org.objectweb.util.browser.core.popup.AdvancedJPopupMenu;
import org.objectweb.util.browser.core.popup.DefaultMenuItemTreeView;
import org.objectweb.util.browser.core.panel.AssociatedPanelImpl;
import org.objectweb.util.browser.core.api.AssociatedContext;
import org.objectweb.util.browser.core.api.AssociatedDropAction;
import org.objectweb.util.browser.core.api.AssociatedIcon;
import org.objectweb.util.browser.core.api.AssociatedInfo;
import org.objectweb.util.browser.core.api.AssociatedMenu;
import org.objectweb.util.browser.core.api.AssociatedPanel;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.api.ExtendedEntry;
import org.objectweb.util.browser.core.api.RootFactory;
import org.objectweb.util.browser.core.api.Role;
import org.objectweb.util.browser.core.dnd.AssociatedDropActionImpl;
import org.objectweb.util.browser.core.icon.AssociatedIconImpl;
import org.objectweb.util.browser.core.info.AssociatedInfoImpl;

/**
 * Centralizes the different customizations
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class AdminCustomization {

    /** The associated tree */
    protected DynamicTree tree_;

    /** The icons map */
    protected AssociatedIcon icons = null;

    /** The panels map */
    protected AssociatedPanel panels = null;

    /** The menus map */
    protected AssociatedMenu menus = null;

    /** The wrappers map */
    protected AssociatedContext wrappers = null;

    /** The DropAction map */
    protected AssociatedDropAction dropAction = null;

    /** The infos map */
    protected AssociatedInfo infos = null;

    /** The associated BrowserProperty */
    protected BrowserProperty properties_ = null;
    
    /** Indicates the target is a tree */
    public static final int TREE_TARGET = 0;
    
    /** Indicates the target is a table */
    public static final int TABLE_TARGET = TREE_TARGET + 1;

    /**
     * Default Constructor
     * @param properties The memory object which contains the properties
     * @param tree The current tree 
     */
    public AdminCustomization(BrowserProperty properties, DynamicTree tree) {
        tree_ = tree;
        properties_ = properties;
        if (properties != null) {
            icons = new AssociatedIconImpl(properties);
            menus = new AssociatedMenuImpl(properties, tree);
            panels = new AssociatedPanelImpl(properties);
            wrappers = new AssociatedContextImpl(properties);
            dropAction = new AssociatedDropActionImpl(properties);
            infos = new AssociatedInfoImpl(properties);
        } else
            System.out.println("The properties param is null !");
    }

    /**
     * Gives the current BrowserProperty. 
     * @return The current BrowserProperty
     */
    public BrowserProperty getBrowserProperty() {
        return properties_;
    }

    /**
     * Tries to find a menu for the given entry.
     * It looks for the menu by contacting the AssociatedMenu instance.
     *   
     * If the given object is an <code>Entry</code> instance:
     * <ul>
     *   <li>1. First, its looks for with the type of the wrapped object by the entry</li>
     *   <li>2. Then, its looks for with the type of the value of the entry</li>
     * </ul> 
     * <ul>
     * <li>1. First, it looks for with the type of the wrapped object by the entry</li>
     * <li>2. Then, it looks for with the type of the value of the entry</li>
     * <li>3. After, it looks for with the type of the wrapper object of the parent of the givent entry</li>
     * <li>4. After, it looks for with the type of the value of the parent of the givent entry</li>
     * <li>5. Finally, it merges the four menus</li>
     * </ul>
     * @return The associated menu
     */
    public JPopupMenu getMenu(Object object, int target) {
        if (object != null) {
            if(target == TREE_TARGET) {
                ExtendedEntry entry = (ExtendedEntry)object;
                AdvancedJPopupMenu objectMenu = menus.getMenu(entry.getWrappedObject(), true, new DefaultMenuItemTreeView(tree_));
                AdvancedJPopupMenu compositeMenu = menus.getMenu(entry.getValue(), new DefaultMenuItemTreeView(tree_));
                ExtendedEntry parentEntry = (ExtendedEntry) tree_.getSelectedEntryParent();
                AdvancedJPopupMenu objectChildMenu = null;
                AdvancedJPopupMenu compositeChildMenu = null;
                if (parentEntry != null) {
                    objectChildMenu = menus.getParentMenu(parentEntry.getWrappedObject());
                    compositeChildMenu = menus.getParentMenu(parentEntry.getValue());
                }
                AdvancedJPopupMenu fusion = new AdvancedJPopupMenu(tree_);
                fusion.add(objectMenu);
                fusion.add(compositeMenu);
                if (objectMenu != null && objectMenu.hasToInheritTreeMenu() || objectMenu == null) {
                    fusion.add(objectChildMenu);
                    fusion.add(compositeChildMenu);
                }
                return fusion;
            } else if(target == TABLE_TARGET) {
                if(ExtendedEntry.class.isAssignableFrom(object.getClass())){
                    ExtendedEntry entry = (ExtendedEntry)object;
                    AdvancedJPopupMenu objectMenu = menus.getMenu(entry.getWrappedObject(), true, new DefaultMenuItemTreeView(tree_, entry));
                    AdvancedJPopupMenu compositeMenu = menus.getMenu(entry.getValue(), new DefaultMenuItemTreeView(tree_, entry));
                    AdvancedJPopupMenu fusion = new AdvancedJPopupMenu(tree_);
                    fusion.add(objectMenu);
                    fusion.add(compositeMenu);
                    return fusion;
                } else {
                    return menus.getMenu(object, new DefaultMenuItemTreeView(tree_, new DefaultEntry(object, new DefaultName(object.toString()),null)));
                }
            }
        }
        return null;
    }

    /**
     * Equals to <code>getMenu(object,TREE_TARGET);</code>.
     * @see AdminCustomization#getMenu(Object object, boolean isTreeMenu);
     */
    public JPopupMenu getMenu(Object object) {
        return getMenu(object,TREE_TARGET);
    }

    /**
     * Tries to find an icon for the given object.
     * It looks for the icon by contacting the AssociatedIcon instance.
     * 
     * If the given object is an <code>Entry</code> instance:
     * <ul>
     *   <li>1. First, its looks for with the type of the wrapped object by the entry</li>
     *   <li>2. Then, its looks for with the type of the value of the entry</li>
     * </ul> 
     * 
     * @param object The object for which an icon may be defined.
     * @param disabledJavaLangObject If true, then java.lang.Object is not consider as a valid entry
     * 
     * @return The associated icon
     */
    public Icon getIcon(Object object){
        if(object != null) {
            Icon icon = null;
            if(ExtendedEntry.class.isAssignableFrom(object.getClass())){
                ExtendedEntry entry = (ExtendedEntry)object;
                icon = icons.getIcon(entry.getWrappedObject(), true);
                if (icon == null)
                    icon = icons.getIcon(entry.getValue());
                return icon;        
            } else
                return icons.getIcon(object);
        }
        return null;        
    }


    /**
     * Tries to find a panel for the given entry.
     * It looks for the panel by contacting the AssociatedPanel instance.
     * <ul>
     * <li>1. First, its looks for with the type of the wrapped object by the entry</li>
     * <li>2. Then, its looks for with the type of the value of the entry</li>
     * </ul>
     * @return The associated icon
     */
    public Panel getPanel(ExtendedEntry entry) {
        if (entry != null) {
            Panel panel = panels.getPanel(entry.getWrappedObject(), true);
            if (panel == null)
                panel = panels.getPanel(entry.getValue());
            return panel;
        }
        return null;
    }

    /**
     * Tries to find a context for the given node.
     * @return The associated context
     */
    public Context getContext(Object node) {
        if (wrappers != null)
            return wrappers.getContext(node);
        return null;
    }

    /**
     * Tries to find an action for the given entry.
     * It looks for the action by contacting the AssociatedDropAction instance.
     * <ul>
     * <li>1. First, its looks for with the type of the wrapped object by the entry</li>
     * <li>2. Then, its looks for with the type of the value of the entry</li>
     * </ul>
     * @return The associated drop action
     */
    public DropAction getDropAction(ExtendedEntry entry) {
        if (entry != null) {
            DropAction action = dropAction.getDropAction(entry.getWrappedObject(), true);
            if (action == null)
                action = dropAction.getDropAction(entry.getValue());
            return action;
        }
        return null;
    }

    /**
     * Tries to find a description of the drop action if this one is defined 
     * It looks for the action by contacting the AssociatedDropAction instance.
     * <ul>
     * <li>1. First, its looks for with the type of the wrapped object by the entry</li>
     * <li>2. Then, its looks for with the type of the value of the entry</li>
     * </ul>
     * @return A description of the Drop action
     */
    public String getDropLabel(ExtendedEntry entry) {
        if (entry != null) {
            String label = dropAction.getDropLabel(entry.getWrappedObject(), true);
            if (label == null)
                label = dropAction.getDropLabel(entry.getValue());
            return label;
        }
        return "";    
    }

    /**
     * Tries to find an Info object for the given entry.
     * It looks for the Info by contacting the AssociatedInfo instance.
     * <ul>
     * <li>1. First, its looks for with the type of the wrapped object by the entry</li>
     * <li>2. Then, its looks for with the type of the value of the entry</li>
     * </ul>
     * @return The associated Info.
     */
    public Info getInfo(ExtendedEntry entry) {
        if (entry != null) {
            Info info = infos.getInfo(entry.getWrappedObject(), true);
            if (info == null)
                info = infos.getInfo(entry.getValue());
            return info;
        }
        return null;
    }
    
    /**
     * Provides the root context depending on the selected role.
     */
    public Entry[] getRoot() {
        Role[] roles = properties_.getInheritedRoles();
        Vector values = new Vector();
        Entry[] entries = null;
        for(int i = 0 ; i < roles.length ; i++) {
            RootFactory rootFactory = properties_.getRoot(roles[i]);
            if(rootFactory!=null){
                entries = rootFactory.newRootContext();
                for(int j = 0 ; j < entries.length ; j++)
                    values.add(entries[j]);
            }
        }
        return (Entry[])values.toArray(new Entry[0]);
    }
    
    /**
     * Removes the property loaded in cache.
     */
    public void clear(){
        icons.clear();
        panels.clear();
        menus.clear();
        wrappers.clear();
        dropAction.clear();
        infos.clear(); 
    }
}
