/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.common;

/** The java API's imports */
import java.util.Vector;
import java.util.List;
import java.util.Iterator;

/**
 * This class gives the inheritance tree of a given classe ordered by interface first and by level.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * @version 0.1
 */
public class ClassesInheritance {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Contains the list of classes and interfaces */
    protected List classes_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor.
     * @param theClass The given you want to obtain the inherit classes
     */
    public ClassesInheritance(Class theClass) {
        classes_ = new Vector();
        if (theClass != null)
            build(theClass);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Indicates if the given class name already exists in the list.
     * @param className The class to check
     * @return The classe which is in the list or null
     */
    protected Class getClass(String className) {
        Iterator it = classes_.iterator();
        while (it.hasNext()) {
            Class c = (Class) it.next();
            if (className.equals(c.getName()))
                return c;
        }
        return null;
    }

    /**
     * Adds the given class to the list.
     * @param c The class to add
     */
    protected void buildClass(Class c) {
        // Adds the class to the given vector
        if (c != null) {
            classes_.add(c);
        }
    }

    /**
     * Adds the given interfaces to the list.
     * If the map previously contained a mapping for this class, the highest level class is kept.
     * <ul>
     * <li>1. Adds the interfaces to the list</li>
     * <li>2. Creates an array containing all the interface extended by the given interfaces</li>
     * <li>3. Recursive call with the created array</li> 
     * </ul>
     * @param interfaces List of interfaces 
     */
    protected void buildInterfaces(Class[] interfaces) {
        // Adds the interfaces to the given vector
        for (int i = 0; i < interfaces.length; i++) {
            Class oldClass = getClass(interfaces[i].getName());
            if (oldClass != null)
                classes_.remove(oldClass);
            classes_.add(interfaces[i]);
        }
        // Creates an array containing all the interface extended by the given interface's array
        Vector allExtendedInterfaces = new Vector();
        for (int i = 0; i < interfaces.length; i++) {
            Class[] extendedInterfaces = interfaces[i].getInterfaces();
            for (int j = 0; j < extendedInterfaces.length; j++) {
                allExtendedInterfaces.add(extendedInterfaces[j]);
            }
        }
        // Recursive call
        if (allExtendedInterfaces.size() != 0)
            buildInterfaces(
                (Class[]) allExtendedInterfaces.toArray(new Class[0]));
    }

    /**
     * Adds the implemented interfaces and the subclasses of the given class. It also adds the given class.
     * <ul>
     * <li>1. Call to buildClass with te given class</li>
     * <li>2. Call to buildInterfaces with the implemented interface of the given class</li>
     * <li>3. Recursive call with the superclass of the given class</li>
     * </ul>
     * @param c The class to analyse
     */
    protected void build(Class c) {
        if (c != null) {
            buildClass(c);
            buildInterfaces(c.getInterfaces());
            build(c.getSuperclass());
        }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Gives the list of inherit classes and interfaces.
     * @return The list containing the implemented interfaces and the subclasses of the class given to the construvtor
     */
    public List getInheritClasses() {
        return classes_;
    }

    // ==================================================================
    //
    // Test class.
    //
    // ==================================================================

    /**
     * Test the class with a java.util.ArrayList object.
     */
    public static void main(String[] args) {
        ClassesInheritance ci = new ClassesInheritance((new java.util.ArrayList()).getClass());
        List list = ci.getInheritClasses();
        Iterator it = list.iterator();
        while (it.hasNext()) {
            System.out.println("=> " + it.next());
        }
    }

}
