/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.common;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Icon;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.api.Decoder;
import org.objectweb.util.browser.core.api.TableConfiguration;

import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;

/**
 * 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class DynamicTable 
     extends JTable 
  implements TableConfiguration {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    protected AdminCustomization custom_;

    protected TableCellRenderer tableCellRenderer_;
        
    protected Decoder decoder_;
        
    //==================================================================
    //
    // Constructor.
    // 
    //==================================================================

    public DynamicTable(Decoder currentDecoder){
        super();
        tableCellRenderer_ = new MyTableCellRenderer();
        decoder_ = currentDecoder;
        addMouseListener(new MyMouseAdapter());
        setColumnSelectionAllowed(true);
        setRowSelectionAllowed(true);
    }

    public DynamicTable(){
        this(null);
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods overriding JTable methods.
    //
    //==================================================================

    public TableCellRenderer getCellRenderer(int row, int col) {
        return tableCellRenderer_; 
    }

    //==================================================================
    //
    // Public methods for TableConfiguration interface.
    //
    //==================================================================

    public void setAdminCustomization(AdminCustomization custom) {    
        custom_ = custom;
    }

    public void setData(String[] headers, Object[][] values){
        setModel(new TableModel(headers, values));                
    }

    //==================================================================
    //
    // Public methods for TableConfiguration interface.
    //
    //==================================================================

    protected class TableModel
            extends AbstractTableModel {
    
        protected String[] headers_;
        protected Object[][] values_; 
    
        public TableModel(String[] headers, Object[][] values){
            headers_ = headers;
            values_ = values;
            if(values_!=null && decoder_!=null){
                for(int i = 0 ; i < values_.length ; i++){
                    for(int j = 0 ; j < values_[i].length ; j++){
                        Object value = getValueAt(i,j);
                        if(value!=null){
                            if(Entry.class.isAssignableFrom(value.getClass())){
                                Entry currentEntry = (Entry)value;
                                Entry newEntry = new DefaultEntry(decoder_.decode(currentEntry.getValue()),new DefaultName(currentEntry.getName().toString()),null);
                                values_[i][j] = newEntry;
                            } else {
                                values_[i][j] = decoder_.decode(value);
                            }
                        }
                    }
                }
            }
        }
    
        /**
         * Overriding methods
         * @see javax.swing.table.TableModel#getColumnName(int)
         */
        public String getColumnName(int column) {
            if(headers_!=null)
                return headers_[column];
            return super.getColumnName(column);
        }

        /**
         * Overriding methods
         * @see javax.swing.table.TableModel#getColumnCount()
         */
        public int getColumnCount() {
            if(headers_!=null)
                return headers_.length;
            return 0;
            }

        /**
         * Overriding methods
         * @see javax.swing.table.TableModel#getRowCount()
         */
        public int getRowCount() {
            if(values_ != null){
                return values_.length;
            }
            return 0; 
        }

        /**
         * Overriding methods
         * @see javax.swing.table.TableModel#getValueAt(int, int)
         */
        public Object getValueAt(int rowIndex, int columnIndex) {
            if(values_ != null)
                return values_[rowIndex][columnIndex];
            return null;
        }
        
        
    }

    /**
     * Mouse Adapter
     * Use to draw Popup Menu
     */
    protected final class MyMouseAdapter extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            popupLayout(e);
        }
        public void mouseReleased(MouseEvent e) {
            popupLayout(e);
        }
        private void popupLayout(MouseEvent e) {
            if (e.isPopupTrigger()) {
                Point point = e.getPoint();
                int row = rowAtPoint(point);
                int col = columnAtPoint(point);
                if((row != -1) && (col != -1)) {
                    Object object = getValueAt(row,col);
                    if (custom_ != null) {
                        JPopupMenu menu = custom_.getMenu(object,AdminCustomization.TABLE_TARGET);
                        if (menu != null) {
                            changeSelection(row,col,false,false);
                            menu.show(DynamicTable.this,(int) (point.getX()),(int) (point.getY()));
                        }
                    }
                } 
            }   
        }
    }

    /**
     * Table Cell Renderer
     * Use to put the icon corresponding to the class 
     * of the Value object of the Entry
     */
    protected final class MyTableCellRenderer extends DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
            if(value != null) {
                if(Entry.class.isAssignableFrom(value.getClass()))            
                    setText(((Entry)value).getName().toString());
                else
                    setText(value.toString());
              
                if (custom_ != null) {
                    Icon icon = custom_.getIcon(value);
                    if (icon != null)
                        setIcon(icon);
                    else
                        setIcon(null);
                }    
            } else {
                setIcon(null);
            }
            return this; 
             
        }

    }


}