/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.gui.lib;

/** The Browser API's imports */
import org.objectweb.util.browser.gui.api.ValidateReport;

/** The java API's imports */
import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.SwingConstants;

import java.awt.Component;
import java.awt.Dimension;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * This class represents the box which allows to choose a file.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class FileChooserBox 
    extends AbstractElementBox {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The file chooser that allow to find the file to load */
    protected JFileChooser fileChooser_;

    /** The chosen file */
    protected File file_;

    /** The value of the label */
    protected String label_;

    /** The textfield for specifying the URL to load */
    protected JTextField urlField_;

    //==================================================================
    //
    // Constructors.
    //
    //==================================================================

    /**
     * Default Constructor
     * @param label The label of the box
     * @param fileChooser The fileChooser to use
     */
    public FileChooserBox(String label, JFileChooser fileChooser){
        this(label,fileChooser,true, true);
    }

    /**
     * Constructor which specifies if the value is mandatory or not
     * @param label The label of the box
     * @param fileChooser The fileChooser to use
     * @param isMandatory If the value is mandatory: TRUE, else: FALSE
     */
    public FileChooserBox(String label, JFileChooser fileChooser, boolean isMandatory){
        this(label,fileChooser, isMandatory, true);
    }    

    /**
     * Constructor which specifies if the value is mandatory or not 
     * @param label The label of the box
     * @param fileChooser The fileChooser to use
     * @param isMandatory If the value is mandatory: TRUE, else: FALSE
     * @param isEditable If the text field can be modify:TRUE else: FALSE 
     */
    public FileChooserBox(String label, JFileChooser fileChooser, boolean isMandatory, boolean isEditable) {
        super(isMandatory);
        fileChooser_ = fileChooser;
        label_ = label;
        add(Box.createHorizontalGlue());
        JLabel browserLabel = new JLabel(label + ": ", SwingConstants.RIGHT);
        browserLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        browserLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(browserLabel);
        add(Box.createHorizontalStrut(5));
        urlField_ = new JTextField();
        urlField_.setEditable(isEditable);
        urlField_.setPreferredSize(new Dimension(200, 20));
        urlField_.setMaximumSize(new Dimension(200, 20));
        add(urlField_);
        add(Box.createHorizontalStrut(5));
        JButton browseButton = new JButton(new BrowseAction(this));
        browseButton.setPreferredSize(new Dimension(20, 20));
        browseButton.setMaximumSize(new Dimension(20, 20));
        add(browseButton);
    }

    // ==================================================================
    //
    //  Public methods for ElementBox.
    //
    // ==================================================================

    /**
     * Validates the content of the ElementBox. Indicates if the different value are acceptables or not.
     * @return The corresponding ValidateReport.
     */
    public ValidateReport validateBox(){
        if(isMandatory_){
            if(urlField_.getText()==null || urlField_.getText().equals(""))
                return new DefaultValidateReport(false,"The \"" + label_ + "\" value is mandatory");
        }
        return new DefaultValidateReport();
    }
    
    /**
     * Returns the Box contains by the ElementBox.
     * @return The Box contains by the ElementBox.
     */
    public Box getBox(){
        return this;
    }
    
    //==================================================================
    //
    // Public methods.
    //
    //==================================================================

    /** Returns the chosen file (on the local file system) or null 
     * if the JChooserBox is not used. 
     */
    public File getFile() {
        return file_;
    }

    /**
     * Returns the specified URL or null if the URL is malformed.
     * @return The specified URL
     */
    public URL getURL() {
        try {
            return new URL(urlField_.getText());            
        } catch (MalformedURLException e) {
            return null;
        }
    }

    //==================================================================
    //
    // Internal classes.
    //
    //==================================================================

    /**
     * Action which 
     */
    protected class BrowseAction extends AbstractAction {

        /** The parent of the panel */
        protected Component parent_;

        /**
         * Default constructor
         */
        protected BrowseAction(Component parent) {
            super("...", null);
            parent_ = parent;
        }

        public void actionPerformed(java.awt.event.ActionEvent ae) {
            int returnVal = fileChooser_.showOpenDialog(parent_);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                file_ = fileChooser_.getSelectedFile();
                try {
                    urlField_.setText(fileChooser_.getSelectedFile().toURL().toString());
                } catch (MalformedURLException e) {
                    System.out.println(getClass().getName() + " Malformed URL Exception !");
                }
            }
        }

    }

}
