/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package DiningPhilosophers.browser;

/** The administration console's imports */
import org.objectweb.util.browser.api.Panel;
import org.objectweb.util.browser.api.TreeView;

/** The CCM Plugin imports */
import org.objectweb.ccm.CORBA.TheRootPOA;

/** The java API's imports */
import java.awt.Color;
import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;

/** The CCM imports */
import org.omg.PortableServer.POA;
import org.omg.Components.Cookie;
import org.omg.Components.InvalidConnection;

/** The dinner demo imports */
import DiningPhilosophers.*;
import DiningPhilosophers.cif.StatusInfoFactory;
import DiningPhilosophers.cif.StatusInfoImpl;

/**
 * Panel to observe a Philosopher.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class PhilosopherPanel
  implements Panel 
{

    // ==================================================================
    //
    // Static bloc.
    //
    // ==================================================================

    // Executed once at the loading of this class
    static
    {
        // Required to register the StatusInfo valuetype factory to the ORB.
        DiningPhilosophers.StatusInfoFactoryHelper.register(new StatusInfoFactory());
    }

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The Panel which contains all the information. */
    protected JPanel compositePanel_ = null;

    /** The PhilosopherPanel. */
    protected DiningPhilosophers.cif.PhilosopherPanel phPanel_ = null;
    
    /** The cookie corresponding to the subscription. */
    protected Cookie cookie_ = null;

    /** The philosopher. */
    protected Philosopher philosopher_ = null;

    /** The StatusInfoConsumer. */
    protected StatusInfoConsumerImpl info_ = null;

    /** The cookie of the Servant. */
    protected byte[] servantId_ = null; 

    // ==================================================================
    //
    // Internal Methods.
    //
    // ==================================================================

    /**
     * Recursive method wich puts all the sub components of the given component as transparent.
     */
    protected void setOpaque(javax.swing.JComponent comp)
    {
        comp.setOpaque(false);
        java.awt.Component[] subComponents = comp.getComponents();
        for(int i = 0 ; i < subComponents.length ; i++)
        {
            setOpaque((javax.swing.JComponent)subComponents[i]);
        }
    }

    // ==================================================================
    //
    // Public methods for Panel interface.
    //
    // ==================================================================

    /**
     * Creates the panel for the selected Philosopher
     */
    public void selected(TreeView treeView) 
    {
        
        // Obtains the selected Philosopher
        philosopher_ = (Philosopher) treeView.getSelectedObject();

        // Creates the composite panel
        compositePanel_ = new JPanel();
        compositePanel_.setBackground(Color.white);

        // Creates the title panel
        JPanel titlePanel = new JPanel();
        titlePanel.setBackground(Color.white);
        titlePanel.add(new javax.swing.JLabel(philosopher_.name() + " panel"));

        // Configures the philosopher panel
        phPanel_ = new DiningPhilosophers.cif.PhilosopherPanel(new StatusInfoImpl(PhilosopherState.from_int(1), philosopher_.name(), 0, false, false));
        javax.swing.border.Border border = javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED);
        phPanel_.setBorder(javax.swing.BorderFactory.createTitledBorder(border, "Observer"));
        setOpaque(phPanel_);
        
        // Configures and feeds the composite panel
        Box box = Box.createVerticalBox(); 
        box.add(Box.createVerticalGlue());
        box.add(titlePanel);
        box.add(Box.createVerticalStrut(10));
        box.add(phPanel_);
        box.add(Box.createVerticalGlue());
        compositePanel_.add(box);

        // Creates and activates a StatusInfoConsumer        
        info_ = new StatusInfoConsumerImpl(phPanel_);
        try
        {
            POA poa = TheRootPOA.getRootPOA();
            servantId_ = poa.activate_object(info_);
            cookie_ = philosopher_.subscribe_info(StatusInfoConsumerHelper.narrow(poa.id_to_reference(servantId_)));
        }
        catch ( java.lang.Exception ex )
        {
            System.err.println( "An exception has been intercepted" );
            ex.printStackTrace();
        }
    }

    /**
     * Returns a panel which contains a title and an observer panel 
     * in order to know the status of the selected philosopher     
     */ 
    public JPanel getPanel()
    {
        return compositePanel_;   
    }

    /**
     * Unregisters the StatusInfoConsumer
     */
    public void unselected(TreeView treeView) 
    {
        try
        {
            philosopher_.unsubscribe_info(cookie_);
        } 
        catch(InvalidConnection e)
        {
            System.err.println("InvalidConnection: " + e.getMessage());   
        }

        try
        {
            TheRootPOA.getRootPOA().deactivate_object(servantId_);
        }
        catch ( java.lang.Exception ex )
        {
            System.err.println( "An exception has been intercepted" );
            ex.printStackTrace();
        }

    }

}
