/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * The class IdlUnion represents an CORBA IDL union value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlUnion object has one child nodes representing union member.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlUnion extends IdlNode implements IdlWritable {      
    protected String[] members;
    protected String[] tags;
    protected transient TypeCode[] memberTc;
    protected transient TypeCode discTc;
    protected String disc;
    protected int index = -1;
    protected int defaultIndex = 0;
    protected transient Any[] anyTags;

    protected IdlUnion() {
        isLeaf = false;
        setUserObject(this);
    }
        
    protected IdlUnion(Any any) {
        this();
        try {
            setNode(any);
        } catch (Exception e) {
            e.printStackTrace();
        } 
    }
        
    protected IdlUnion(TypeCode tc) {
        this();
        setNode(tc);
    }
    
    protected void setNode(TypeCode tc) {
        try {
            this.tc = tc;
            if (tc.kind().value() == TCKind._tk_union) { 
                id = tc.id();
                type = "union " + id;
                init(tc);
                value = tags[index];
                IdlNode m = create(memberTc[index]);
                m.field = members[index];
                removeAllChildren();
                add(m);
            } else if (tc.kind().value() == TCKind._tk_alias) {
                id = tc.id();
                type = id;
                TypeCode stc = tc.content_type();
                while (stc.kind().value() == TCKind._tk_alias) {
                    stc = stc.content_type();
                }
                if (stc.kind().value() != TCKind._tk_union) {
                    throw new RuntimeException("Union type expected");
                }
                init(stc);
                value = tags[index];
                IdlNode m = create(memberTc[index]);
                m.field = members[index];
                removeAllChildren();
                add(m);
            } else {
                throw new RuntimeException("Union type expected");
            } 
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    protected void init(TypeCode tc) {
        try {
            if (members == null) {
                members = getMembers(tc);               
                discTc = tc.discriminator_type();
                memberTc = new TypeCode[members.length];
                tags = new String[members.length];
                anyTags = new Any[members.length];
                for (int i = 0; i < members.length; i++) {
                    memberTc[i] = tc.member_type(i);
                    tags[i] = getDiscriminatorLabel(tc.member_label(i));
                    anyTags[i] = tc.member_label(i);
                }
                defaultIndex = tc.default_index();
                if (defaultIndex < 0) {
                    // If there is no explicit default, use the first member
                    defaultIndex = 0;
                } else {
                    anyTags[defaultIndex] = getDefaultValue(discTc);
                }
                index = defaultIndex;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }        

    protected void setNode(Any any) {
        try {
            setNode(any.type());                
                
            org.omg.CORBA.portable.InputStream in = any.create_input_stream();
            // first read the discriminator into an any
            Any discAny = orb.create_any();
            discAny.type(discTc);
            discAny.read_value(in, discTc);
            
            // obtain the tag label for the discriminator
            String label = getDiscriminatorLabel(discAny);             
            if (label == null) {
                throw new RuntimeException("Uninitialized union");
            }
            
            // find which index belongs to the discriminator label
            index = defaultIndex;
            for (int i = 0; i < tags.length; i++) {
                if (label.equals(tags[i])) {
                    index = i;
                    break;
                }
            }

            // now read the member from the input stream
            Any memberAny = orb.create_any();
            memberAny.read_value(in, memberTc[index]);
            IdlNode m = create(memberAny);
            m.field = members[index];
            value = tags[index];
            removeAllChildren();
            add(m);
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    /**
     * Return the list of member names for this union.
     *
     * @return The list of member names for this union.
     */    
    public String[] getMembers() {
        return members;
    }
    
    /**
     * Return the list of tag names for this union.
     *
     * @return The list of tag names for this union.
     */
    public String[] getTags() {
        return tags;
    }

    /**
     * Return the current tag name for this union.
     *
     * @return The current tag name for this union.
     */
    public String getTag() {
        return tags[index];
    }
    
    /**
     * Return the current member name for this union.
     *
     * @return The current member name for this union.
     */
    public String getMemberName() {
        return members[index];
    }

    /**
     * Return the current meber as an IdlNode for this union.
     *
     * @return The current meber as an IdlNode for this union.
     */
    public IdlNode getMember() {
        try {
            return (IdlNode)getChildAt(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;    
    }       
        
    /**
     * Return the current tag index number for this union.
     *
     * @return The current tag index number for this union.
     */
    public int getIndex() {
        return index;
    } 

    /**
     * Sets the curent union member from the given tag index number.
     *
     * @param The tag index number for this union.
     */
    public void setIndex(int i) {
        try {
            if (i < 0 || i >= tags.length) {
                throw new RuntimeException("index out of range");
            }
            value = tags[i];
            if (value.equals("default")) {
                index = defaultIndex;
            } else {
                index = i;
            }
            IdlNode n = IdlNode.create(memberTc[index]);
            n.field = members[index];
            removeAllChildren();
            add(n);
        } catch (Throwable t) {
            t.printStackTrace();
        }              
    } 
    
    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {
            if (index < 0) {
                throw new RuntimeException("Uninitialized union");
            }
            Any any = orb.create_any();
            any.type(tc);
            org.omg.CORBA.portable.OutputStream out = any.create_output_stream();
            // write the discriminator tag value for the current index
            anyTags[index].write_value(out);
            // write the member
            ((IdlNode)getChildAt(0)).toAny().write_value(out);
            any.read_value(out.create_input_stream(), tc);
            return any;
        } catch (Throwable e) {
            e.printStackTrace();
        }
        return null;
    }
    
    /**
     * Returns the label tag contained in an any as a string.
     * By convention, an octet with value 0 is used as the default label.
     */                
    private String getDiscriminatorLabel(Any any) {
        try {
            switch (any.type().kind().value()) {
                case TCKind._tk_short: {
                    return "" + any.extract_short();
                }
                case TCKind._tk_ushort: {
                    return "" + any.extract_ushort();
                }
                case TCKind._tk_long: {
                    return "" + any.extract_long();
                }
                case TCKind._tk_ulong: {
                    return "" + any.extract_ulong();
                }
                case TCKind._tk_boolean: {
                    return "" + any.extract_boolean();
                }
                case TCKind._tk_char: {
                    return "" + any.extract_char();
                }
                case TCKind._tk_wchar: {
                    return "" + any.extract_wchar();
                }        
                case TCKind._tk_octet: {
                    String v = "" + any.extract_octet();
                    if (v.equals("0")) {         
                        return "default";
                    } else {
                        return v;
                    }
                }
                case TCKind._tk_enum: {
                    org.omg.CORBA.portable.InputStream in = any.create_input_stream();
                    int v = in.read_ulong();
                    return any.type().member_name(v);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        try {
            ((IdlNode)getChildAt(0)).write(os);                                 
        } catch (Exception e) {
            e.printStackTrace();
        } 
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        try {
            ((IdlNode)getChildAt(0)).read(is);                                 
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }
        
    /**
     * Find a valid discriminitor value for the default label. This value is the first value
     * in the range of possible discriminator values that is not used as a tag.
     * Note that for enumerations there is no possible default value if all enumerators are used as tags.
     * When the value is found, it is returned as an any.
     */
    private Any getDefaultValue(TypeCode dtc) {
        try {
            Any any = orb.create_any();
            any.type(dtc);

            switch (dtc.kind().value()) {
                case TCKind._tk_short: {
                    for (int i = 0; i < 65536; i++) {
                        if (!isLabel("" + i)) {
                            any.insert_short((short)i);
                            return any;
                        }
                    }
                    throw new RuntimeException("no valid default value found");
                }
                case TCKind._tk_ushort: {
                    for (int i = 0; i < 65536; i++) {
                        if (!isLabel("" + i)) {
                            any.insert_ushort((short)i);
                            return any;
                        }
                    }
                    throw new RuntimeException("no valid default value found");
                }
                case TCKind._tk_long: {
                    for (int i = 0; i < 2147483647; i++) {
                        if (!isLabel("" + i)) {
                            any.insert_long(i);
                            return any;
                        }
                    }
                    throw new RuntimeException("no valid default value found");
                }
                case TCKind._tk_ulong: {
                    for (int i = 0; i < 2147483647; i++) {
                        if (!isLabel("" + i)) {
                            any.insert_ulong(i);
                            return any;
                        }
                    }
                    throw new RuntimeException("no valid default value found");
                }
                case TCKind._tk_boolean: {
                    if (!isLabel("TRUE")) {
                        any.insert_boolean(true);
                        return any;
                    } else {
                        any.insert_boolean(false);
                        return any;
                    }
                }
                case TCKind._tk_char: {
                    for (int i = 0; i < 256; i++) {
                        if (!isLabel(new Character((char)i).toString())) {
                            any.insert_char((char)i);
                            return any;
                        }
                    }
                    throw new RuntimeException("no valid default value found");
                }
                case TCKind._tk_wchar: {
                    for (int i = 0; i < 256; i++) {
                        if (!isLabel(new Character((char)i).toString())) {
                            any.insert_wchar((char)i);
                            return any;
                        }
                    }
                    throw new RuntimeException("no valid default value found");
                }        
                case TCKind._tk_enum: {
                    String[] eMembers = new String[dtc.member_count()];
                    for (int i = 0; i < eMembers.length; i++) {
                        eMembers[i] = dtc.member_name(i);
                    }
                    for (int i = 0; i < eMembers.length; i++) {
                        if (!isLabel(eMembers[i])) {
                            org.omg.CORBA.portable.OutputStream out = any.create_output_stream();
                            out.write_ulong(i);
                            any.read_value(out.create_input_stream(), dtc);
                            return any;
                        }
                    }
                    throw new RuntimeException("no valid default value found");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
    
    private boolean isLabel(String label) {
        for (int i = 0; i < tags.length; i++) {
            if (tags[i].equals(label)) {
                return true;
            }
        }
        return false;
    }
    
    private String[] getMembers(TypeCode mtc) {
        try {
            String[] members = new String[mtc.member_count()];
            for (int i = 0; i < members.length; i++) {
                members[i] = mtc.member_name(i);
            }
            return members;
        } catch (Exception e) {
        }
        return null;
    }
    
    // XML section

    /**
     * Create an IdlUnion from an Xml representation.
     *
     * The XML format example
     * <pre>
     * &lt;union id="IDL:example/MyEnumUnion:1.0"&gt;
     *  &lt;tag&gt;blue&lt;/tag&gt;
     *  &lt;member name="m1"&gt;
     *   &lt;string&gt;my member value&lt;/string&gt;
     *  &lt;/member&gt;
     * &lt;/struct&gt;
     * </pre>
     *
     * @param xml The XML string from which to create an IdlUnion instance.
     */
    public IdlUnion(String xml) {
        this(XmlNode.getNode(xml));
    }

    IdlUnion(Node n) {
        this();
        try {
            id = XmlNode.getId(n);
            setNode(XmlNode.type(id));
    
            Node[] nodes = XmlNode.childElements(n);
            if (nodes.length != 2) {
                throw new RuntimeException("Tag and member expected.");
            }
    
            if (!nodes[0].getNodeName().toUpperCase().equals("TAG")) {        
                throw new RuntimeException("Tag expected.");
            }
            
            String tag = XmlNode.getText(nodes[0]);
            for (int i = 0; i < tags.length; i++) {
                if (tags[i].equals(tag)) {
                    index = i;
                    break;
                }
            }
            if (index < 0) {
                throw new RuntimeException("invalid tag: " + tag);
            }
    
            if (!nodes[1].getNodeName().toUpperCase().equals("MEMBER")) {        
                throw new RuntimeException("Member expected.");
            }
    
            String name = XmlNode.getName(nodes[1]);        
            if (!name.equals(members[index])) {
                throw new RuntimeException("tag does not match member name: " + name);
            }
            value = tag;
            
            IdlNode m = XmlNode.getIdlNode(memberTc[index], XmlNode.firstChildElement(nodes[1]));
            m.setField(name);
            removeAllChildren();
            add(m);        
        } catch (Exception e) {
            throw new RuntimeException(e.toString());
        }
    }       
    
    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        write(this, w);
    }

    public static void write(IdlUnion n, IdlWriter w) {
        w.write_start_union(n.getId());
        w.write_union_tag(n.getTag());
        w.write_start_member(n.getMemberName());
        XmlNode.write(n.getMember(), w);
        w.write_end_member();
        w.write_end_union();
    }
}