/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cidl.api;

// Package dependencies.
import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.ConstantDecl;
import org.objectweb.corba.ast.api.EnumDecl;
import org.objectweb.corba.ast.api.StructDecl;
import org.objectweb.corba.ast.api.UnionDecl;
import org.objectweb.corba.ast.api.TypeRefWithContent;
import org.objectweb.corba.ast.api.AbstractInterfaceDecl;
import org.objectweb.corba.ast.api.LocalInterfaceDecl;
import org.objectweb.corba.ast.api.InterfaceDecl;
import org.objectweb.corba.ast.api.ValueDecl;
import org.objectweb.corba.ast.api.ValueBoxDecl;
import org.objectweb.corba.ast.api.ExceptionDecl;
import org.objectweb.corba.ast.api.AliasDecl;
import org.objectweb.corba.ast.api.AnyValue;
import org.objectweb.corba.ast.api.Scope;
import org.objectweb.corba.ast.api.AttributeDecl;
import org.objectweb.corba.ast.api.OperationDecl;
import org.objectweb.corba.ast.api.TypeRef;
import java.util.Set;

/**
 * This interface allows you to get classes dependencies for an IDL2 declaration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public interface IDL2DeclarationDependencies
{
    /**
     * Get Constant dependencies class files.
     *
     * @param aconst - Get dependencies from this constant.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getConstantDependencies(ConstantDecl aconst,
                            Set visited);

    /**
     * Get Enum dependencies class files.
     *
     * @param enum - Get dependencies from this enumeration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getEnumDependencies(EnumDecl enum,
                        Set visited);

    /**
     * Get Struct dependencies class files.
     *
     * @param struct - Get dependencies from this structure.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getStructDependencies(StructDecl struct,
                          Set visited);

    /**
     * Get Union dependencies class files.
     *
     * @param union - Get dependencies from this union.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getUnionDependencies(UnionDecl union,
                         Set visited);

    /**
     * Get Abstract Interface dependencies class files.
     *
     * @param itf - Get dependencies from this abstrat interface.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getAbstractInterfaceDependencies(AbstractInterfaceDecl itf,
                                     Set visited);

    /**
     * Get Local Interface dependencies class files.
     *
     * @param itf - Get dependencies from this local interface.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getLocalInterfaceDependencies(LocalInterfaceDecl itf,
                                  Set visited);

    /**
     * Get Interface dependencies class files.
     *
     * @param itf - Get dependencies from this interface.
     * @param server - If true, include server class files.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getInterfaceDependencies(InterfaceDecl itf,
                             boolean server,
                             Set visited);

    /**
     * Get Value Type dependencies class files.
     *
     * @param value - Get dependencies from this value type.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getValueTypeDependencies(ValueDecl value,
                             Set visited);

    /**
     * Get Value Box dependencies class files.
     *
     * @param box - Get dependencies from this value box.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getValueBoxDependencies(ValueBoxDecl box,
                            Set visited);

    /**
     * Get Exception dependencies class files.
     *
     * @param ex - Get dependencies from this exception.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getExceptionDependencies(ExceptionDecl ex,
                             Set visited);

    /**
     * Get Any Value dependencies class files.
     *
     * @param any - Get dependencies from this Any Value.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getAnyValueDependencies(AnyValue any,
                            Set visited);

    /**
     * Get Alias dependencies class files.
     *
     * @param alias - Get dependencies from this Alias.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getAliasDependencies(AliasDecl alias,
                         Set visited);

    /**
     * Get Attribute dependencies class files.
     *
     * @param att - Get dependencies from this attribute.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getAttributeDependencies(AttributeDecl att,
                             Set visited);

    /**
     * Get Operation dependencies class files.
     *
     * @param op - Get dependencies from this operation.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getOperationDependencies(OperationDecl op,
                             Set visited);

    /**
     * Get scope contents dependencies.
     *
     * @param scope - Get dependencies for scope contents.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getContentsDependencies(Scope scope,
                            Set visited);

    /**
     * Get dependencies for an IDL2 declaration.
     *
     * @param decl - Get dependencies for this declaration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getDeclarationDependencies(Declaration decl,
                               Set visited);

    /**
     * Get Type Reference dependencies class files.
     *
     * @param type - Get dependencies from this type reference.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getTypeRefDependencies(TypeRef type,
                           Set visited);

    /**
     * Get TypeRefWithContent dependencies class files.
     *
     * @param type - Get dependencies from this TypeRefWithContent.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getTypeRefWithContentDependencies(TypeRefWithContent type,
                                      Set visited);
}
