/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.StorageTypeStateMemberDecl;
import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;

/** To access Java AST. */
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;

/** Others. */
import org.objectweb.corba.generator.common.lib.GenerationException;
import org.objectweb.corba.generator.translator.idl2java.api.PSDL_JavaTranslator;


/**
 * This class generates the mapping for a State Member.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */
public class StateMemberMapping
  implements org.objectweb.corba.generator.psdl.api.StateMemberMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The State member declaration. */
    private StorageTypeStateMemberDecl state_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param state - The State Member to map.
     */
    public StateMemberMapping(StorageTypeStateMemberDecl state)
    {
        // Init internal state
        state_ = state;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Map a Storage Type State Member.
     *
     * @param translator - A Java utility class to translate types.
     * @param obj - The StorageType mapping object.
     * @param finalAccessor - True if accessors must be final.
     */
    public void
    toJavaAccessors(PSDL_JavaTranslator translator,
                    InterfaceObject obj,
                    boolean finalAccessor)
    {
        MethodObject method = null;
        ParameterObject param = null;

        if (PSDLTools.isAbstractStorageTypeReference(state_.getType()))
        {
            // Add the read only accessor method returning an incarnation
            method = new MethodObjectImpl();
            method.addComment("Get the "+state_.getName()+" value.");
            method.addComment(" ");
            method.addComment("@return The "+state_.getName()+" value.");
            method.setName( state_.getName() );
            method.setReturnType( translator.RefToJava(state_.getType()) );
            method.setFinal( finalAccessor );
            method.getImpl().setMacro("REF_GET_METHOD");
            method.getImpl().addContextValue("st_type", translator.RefToJava(state_.getType()) );
            method.getImpl().addContextValue("state", state_.getName());
            obj.addMethod(method);

            // Add the read only accessor method returning a reference (pid)
            method = new MethodObjectImpl();
            method.addComment("Get the "+state_.getName()+" value.");
            method.addComment(" ");
            method.addComment("@param yr - Use to have a different signature.");
            method.addComment(" ");
            method.addComment("@return The "+state_.getName()+" value.");
            method.setName( state_.getName() );
            method.setReturnType( translator.toJava(state_.getType()) );
            method.setFinal( finalAccessor );
            param = new ParameterObjectImpl();
            param.setName("yr");
            param.setType("org.omg.CosPersistentState.YieldRef");
            method.addParameter(param);
            method.getImpl().setMacro("GET_METHOD");
            method.getImpl().addContextValue("var", "_"+state_.getName()+"_state");
            obj.addMethod(method);

            if (!state_.isReadonly())
            {
                // Add the modifier method with an incarnation parameter
                method = new MethodObjectImpl();
                method.addComment("Set the "+state_.getName()+" value.");
                method.addComment(" ");
                method.addComment("@param new_one - The value to set.");
                method.setName( state_.getName() );
                method.setReturnType("void");
                method.setFinal( finalAccessor );
                param = new ParameterObjectImpl();
                param.setName("new_one");
                param.setType( translator.RefToJava(state_.getType()) );
                method.addParameter(param);
                method.getImpl().setMacro("REF_SET_METHOD");
                method.getImpl().addContextValue("var", "_"+state_.getName()+"_state");
                method.getImpl().addContextValue("value", "new_one");
                obj.addMethod(method);

                // Add the modifier accessor method with a reference parameter (pid)
                method = new MethodObjectImpl();
                method.addComment("Set the "+state_.getName()+" value.");
                method.addComment(" ");
                method.addComment("@param new_one_ref - The value reference to set.");
                method.setName( state_.getName() );
                method.setReturnType("void");
                method.setFinal( finalAccessor );
                param = new ParameterObjectImpl();
                param.setName("new_one_ref");
                param.setType( translator.toJava(state_.getType()) );
                method.addParameter(param);
                method.getImpl().setMacro("SET_METHOD");
                method.getImpl().addContextValue("var", "_"+state_.getName()+"_state");
                method.getImpl().addContextValue("value", "new_one_ref");
                obj.addMethod(method);
            }
        }
        else
        {
            // Add the read only accessor method
            method = new MethodObjectImpl();
            method.addComment("Get the "+state_.getName()+" value.");
            method.addComment(" ");
            method.addComment("@return The "+state_.getName()+" value.");
            method.setName( state_.getName() );
            method.setReturnType( translator.toJava(state_.getType()) );
            method.setFinal( finalAccessor );
            method.getImpl().setMacro("GET_METHOD");
            method.getImpl().addContextValue("var", "_"+state_.getName()+"_state");
            obj.addMethod(method);

            if (!state_.isReadonly())
            {
                // Add the modifier method
                method = new MethodObjectImpl();
                method.addComment("Set the "+state_.getName()+" value.");
                method.addComment(" ");
                method.addComment("@param new_one - The value to set.");
                method.setName( state_.getName() );
                method.setReturnType("void");
                method.setFinal( finalAccessor );
                param = new ParameterObjectImpl();
                param.setName("new_one");
                param.setType( translator.toJava(state_.getType()) );
                method.addParameter(param);
                method.getImpl().setMacro("SET_METHOD");
                method.getImpl().addContextValue("var", "_"+state_.getName()+"_state");
                method.getImpl().addContextValue("value", "new_one");
                obj.addMethod(method);

                if ( !PSDLTools.isImmutable(state_.getType())
                     && !PSDLTools.isAbstractStorageType(state_.getType()) )
                {
                    // Add the read / update accessor method
                    method = new MethodObjectImpl();
                    method.addComment("Update the "+state_.getName()+" value.");
                    method.addComment(" ");
                    method.addComment("@param fu - ForUpdate.");
                    method.setName( state_.getName() );
                    method.setReturnType( translator.toJava(state_.getType()) );
                    method.setFinal( finalAccessor );
                    param = new ParameterObjectImpl();
                    param.setName("fu");
                    param.setType("org.omg.CosPersistentState.ForUpdate");
                    method.addParameter(param);
                    method.getImpl().setMacro("GET_UPDATE_METHOD");
                    method.getImpl().addContextValue("var", "_"+state_.getName()+"_state");
                    obj.addMethod(method);
                }
            }
        }
    }

    /**
     * Map a Read Only Storage Type State Member to a user non-available modifier.
     *
     * @param translator - A Java utility class to translate types.
     * @param clazz - The StorageType mapping object.
     */
    public void
    toJavaReadOnlyStateMemberModifier(
            PSDL_JavaTranslator translator,
            ClassObject clazz)
    {
        MethodObject method = null;
        ParameterObject param = null;

        method = new MethodObjectImpl();
        method.addComment("Set the "+state_.getName()+" value.");
        method.addComment(" ");
        method.addComment("@param initial_value - The value to set.");
        method.setName( "set_" + state_.getName() );
        method.setReturnType("void");
        method.setFinal(true);
        param = new ParameterObjectImpl();
        param.setName("initial_value");
        param.setType( translator.toJava(state_.getType()) );
        method.addParameter(param);
        method.getImpl().setMacro("SET_METHOD");
        method.getImpl().addContextValue("var", "_"+state_.getName()+"_state");
        method.getImpl().addContextValue("value", "initial_value");
        clazz.addMethod(method);
    }

    /**
     * Map a Storage Type State Member to an attribute.
     *
     * @param translator - A Java utility class to translate types.
     * @param clazz - The StorageType mapping object.
     * @param st - The StorageType declaration.
     */
    public void
    toJavaAttribute(PSDL_JavaTranslator translator,
                    ClassObject clazz,
                    org.objectweb.corba.ast.api.StorageTypeDecl st)
    {
        AttributeObject att = null;
        String marshalled_type = null;

        att = new AttributeObjectImpl();
        att.addComment("The "+state_.getName()+" state.");
        att.setName("_"+state_.getName()+"_state");
        att.setType( translator.toJava(state_.getType()) );
        att.setModifier( ModifierKindImpl.mk_private );
        clazz.addAttribute(att);

        // Add the mapping attribute if necessary
        marshalled_type = PSDLMappingTools.getMarshalledType(st, state_);
        if (marshalled_type != null)
        {
            // Add a XXX_marshalled attribute
            att = new AttributeObjectImpl();
            att.addComment("The "+state_.getName()+" marshalled state.");
            att.setName("_"+state_.getName()+"_marshalled");
            att.setType(marshalled_type);
            att.setModifier( ModifierKindImpl.mk_private );
            clazz.addAttribute(att);
        }
    }

    /**
     * Map a state member to a java parameter.
     *
     * @param translator - A Java utility class to translate types.
     * @param method - The method that will contain parameters.
     **/
    public void
    toJavaParameter(PSDL_JavaTranslator translator,
                    MethodObject method)
    {
        ParameterObject param = null;

        // Create the corresponding parameter
        param = new ParameterObjectImpl();
        param.setName(state_.getName());
        param.setType( translator.toJava(state_.getType()) );
        method.addParameter(param);
    }

    /**
     * Map a state list (string[]) to ParameterObject for the java AST.
     *
     * @param generator - The PSDL to Java generator.
     * @param states - The state list.
     * @param method - The method that will contain parameters.
     * @param ast - The storage type containing states.
     **/
    public static void
    mapStateListToParameters(org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator,
                             String[] states,
                             MethodObject method,
                             AbstractStorageTypeDecl ast)
    throws GenerationException
    {
        StorageTypeStateMemberDecl state = null;
        ParameterObject param = null;

        for (int i=0; i<states.length; i++)
        {
            // Find the state in the current storageType
            state = (StorageTypeStateMemberDecl)generator.getDeclaration(ast, states[i]);

            if (state == null)
            {
                AbstractStorageTypeDecl[] implemented = null;

                // Find in the implemented classes
                implemented = ast.getAllImplementedAbstractStorageTypes().getAbstractStorageTypes();
                if(implemented.length > 0)
                {
                    int j = 0;
                    while( (j<implemented.length) && (state == null) )
                    {
                        state = (StorageTypeStateMemberDecl)generator.getDeclaration(implemented[j], states[i]);
                        j++;
                    }
                }
                if(state == null)
                {
                    String msg = "State Member '"+states[i]+"' not found!";
                    throw new GenerationException(msg);
                }
            }

            // Create the corresponding parameter
            org.objectweb.corba.generator.psdl.api.StateMemberMapping smm
                = new StateMemberMapping(state);
            smm.toJavaParameter(generator.getTranslator(), method);
        }
    }

}
