# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# ====================================================================
#
#  This class provides methods to check all the constraints
#  defined in the UML Profile for CORBA Components (Version 3.1).
#
# ====================================================================

import CORBA2Checker;
import ConstraintException;

class CORBA3Checker (CORBA2Checker.CORBA2Checker)
{

    # =================================================================
    #   Empty constructor for the CORBA3Checker.
    # =================================================================
    proc __CORBA3Checker__(self)
    {
        self.__CORBA2Checker__();
    }


    # =================================================================
    #   Prints an error message.
    #   stereotype - The stereotype which doesn't respect the constraint
    #   cause - The name of the element which causes the exception
    #   section - The corresponding section of the specification
    #   message - The error message
    # =================================================================
    proc error(stereotype, cause, section, message) {
        throw ConstraintException.ConstraintException (stereotype, cause, "UML Profile for CORBA Components (Version 3.1)", section, message);
    }


    # =================================================================
    #   4.1.2.1 - Component
    # =================================================================
    proc checkCORBAComponent (self, element)
    {
        self.checkCORBAInterface(element);
        for feat in element.getFeature().toArray() {
            if (feat._is_a(org.omg.uml.core.BehavioralFeature)) {
                error("CORBAComponent", element.getName(), "4.1.2.1", "A CORBAComponent cannot own operations.");
            }
        }
        correctInheritance = true;
        numberInheritComponent = 0;
        for g in element.getGeneralization().toArray() {
            if (self.umlOperations.isStereotyped(g.getParent(), "CORBAInterface")) {
                if (!self.umlOperations.isStereotyped(g, "CORBASupports")) {
                    error("CORBAComponent", element.getName(), "4.1.2.1", "Each CORBAComponent inheritance from a CORBAInterface must be stereotyped CORBASupports");
                }
            } else if (self.umlOperations.isStereotyped(g.getParent(), "CORBAComponent")) {
                numberInheritComponent = numberInheritComponent + 1 ;
            } else {
                correctInheritance = false;
            }
        }
        if (!correctInheritance || numberInheritComponent > 1) {
            error("CORBAComponent", element.getName(), "4.1.2.1", "A CORBAComponent can only inherit from one CORBAComponent or CORBAInterface.");
        }
    }


    # =================================================================
    #   4.1.2.2 - Facet and Receptacle
    # =================================================================
    proc checkCORBAPort (self, element)
    {
        interfaceSide = "notset";
        componentSide = "notset";        
        for connect in element.getConnection().toArray() {
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAInterface")
                || self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAHome")) {
                interfaceSide = connect;
            }
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAComponent")) {
                componentSide = connect;
            }
        }
        if (element.getConnection().size() != 2) {
            error("CORBAPort", element.getName(), "4.1.2.2", "It's a binary association.");
        }
        if (interfaceSide == "notset" || componentSide == "notset") {
            error("CORBAPort", element.getName(), "4.1.2.2", "It's an association between a CORBAComponent and a CORBAInterface.");
        }
        if (componentSide.getAggregation() != org.omg.uml.data_types.AggregationKindEnum.forName("ak_composite")) {
            error("CORBAPort", element.getName(), "4.1.2.2", "The CORBAComponent side is a composition.");
        }
        if (interfaceSide.getName() == "" || interfaceSide.getName() == JAVA.null || interfaceSide.getName().length == 0) {
            error("CORBAPort", element.getName(), "4.1.2.2", "The CORBAInterface role name must be defined.");
        }
    }


    # =================================================================
    #   4.1.2.2 - Facet and Receptacle - CORBAProvides
    # =================================================================
    proc checkCORBAProvides (self, element)
    {
        self.checkCORBAPort(element);
        for connect in element.getConnection().toArray() {
            lower = 0;
            upper = 0;
            for range in connect.getMultiplicity().getRange().toArray() {
                lower = range.getLower();
                upper = range.getUpper();
            }
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAInterface")
                || self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAHome")) {
                if (lower != 1 || upper != 1) {
                    error("CORBAProvides", connect.getParticipant().getName(), "4.1.2.2", "The CORBAInterface side cardinality must be 1.");
                }
            }
        }
    }


    # =================================================================
    #   4.1.2.2 - Facet and Receptacle - CORBAUses
    # =================================================================
    proc checkCORBAUses (self, element)
    {
        self.checkCORBAPort(element);
        nameUses = "noname";
        for connect in element.getConnection().toArray() {
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAInterface")
                || self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAHome")) {
                lower = 0;
                upper = 0;
                for range in connect.getMultiplicity().getRange().toArray() {
                    lower = range.getLower();
                    upper = range.getUpper();
                }
                multiple = "no";
                for tv in element.getTaggedValue().toArray() {
                    if (tv.getType().getTagType() == "CORBAIsMultiple") {
                        multiple = tv.getDataValue();
                    }
                }
                nameUses = element.getName();
                if (nameUses == JAVA.null) {
                    nameUses = connect.getParticipant().getName();
                }
                if (multiple != "yes" && multiple != "no") {
                    error("CORBAUses", nameUses, "4.1.2.2", "The CORBAIsMultiple tagged value has an incorrect value. \"Multiple\" value can be 'yes' or 'no'.");
                }
                if (multiple == "yes" && (lower != 1 || upper ==0 || upper == 1)) {
                    error("CORBAUses", nameUses, "4.1.2.2", "The CORBAInterface side cardinality must greater than one for multiple receptacles.");
                }
                if (multiple == "no" && (lower != 1 || upper != 1)) {
                    error("CORBAUses", nameUses, "4.1.2.2", "The CORBAInterface side cardinality must be 1 for simple receptacle.");
                }
            }
        }
    }


    # =================================================================
    #   4.1.2.3 - Event
    # =================================================================
    proc checkCORBAEvent (self, element)
    {
        self.checkCORBAValue(element);
    }


    # =================================================================
    #   4.1.2.3 - EventPort
    # =================================================================
    proc checkCORBAEventPort (self, element)
    {
        eventSide = "notset";
        componentSide = "notset";
        for connect in element.getConnection().toArray() {
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAEvent")) {
                eventSide = connect;
            }
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAComponent")) {
                componentSide = connect;
            }
        }
        if (element.getConnection().size() != 2) {
            error("CORBAEventPort", element.getName(), "4.1.2.3", "It's a binary association.");
        }
        if (eventSide == "notset" || componentSide == "notset") {
            error("CORBAEventPort", element.getName(), "4.1.2.3", "It's an association between a CORBAComponent and a CORBAEvent.");
        }
        if (componentSide.getAggregation() != org.omg.uml.data_types.AggregationKindEnum.forName("ak_composite")) {
            error("CORBAEventPort", element.getName(), "4.1.2.3", "The CORBAComponent side is a composition.");
        }
        if (eventSide.getName() == "" || eventSide.getName() == JAVA.null || eventSide.getName().length() == 0) {
            error("CORBAEventPort", element.getName(), "4.1.2.3", "The CORBAEvent side role name must be defined.");
        }
    }


    # =================================================================
    #   4.1.2.3 - CORBAConsumes
    # =================================================================
    proc checkCORBAConsumes (self, element)
    {
        self.checkCORBAEventPort(element);
        lower = 0;
        upper = 0;
        connect = element.getConnection();
        if (self.umlOperations.isStereotyped(connect.get(0).getParticipant(), "CORBAEvent")) {
            lower = connect.get(0).getMultiplicity().getRange().get(0).getLower();
            upper = connect.get(0).getMultiplicity().getRange().get(0).getUpper();
        } else {
            lower = connect.get(1).getMultiplicity().getRange().get(0).getLower();
            upper = connect.get(1).getMultiplicity().getRange().get(0).getUpper();
        }
        if (lower != 1 || upper != 1) {
            error("CORBAConsumes", element.getName(), "4.1.2.3", "The CORBAEvent side cardinality must be 1.");
        }
    }


    # =================================================================
    #   4.1.2.3 - CORBAEmits
    # =================================================================
    proc checkCORBAEmits (self, element)
    {
        self.checkCORBAEventPort(element);
        lower = 0;
        upper = 0;
        connect = element.getConnection();
        if (self.umlOperations.isStereotyped(connect.get(0).getParticipant(), "CORBAEvent")) {
            lower = connect.get(0).getMultiplicity().getRange().get(0).getLower();
            upper = connect.get(0).getMultiplicity().getRange().get(0).getUpper();
        } else {
            lower = connect.get(1).getMultiplicity().getRange().get(0).getLower();
            upper = connect.get(1).getMultiplicity().getRange().get(0).getUpper();
        }
        if (lower != 1 || upper != 1) {
            error("CORBAEmits", element.getName(), "4.1.2.3", "The CORBAEvent side cardinality must be 1.");
        }
    }


    # =================================================================
    #   4.1.2.3 - CORBAPublishes
    # =================================================================
    proc checkCORBAPublishes (self, element)
    {
        self.checkCORBAEventPort(element);
        lower = 0;
        upper = 0;
        connect = element.getConnection();
        if (self.umlOperations.isStereotyped(connect.get(0).getParticipant(), "CORBAEvent")) {
            upper = connect.get(0).getMultiplicity().getRange().get(0).getUpper();
        } else {
            upper = connect.get(1).getMultiplicity().getRange().get(0).getUpper();
        }
        if (upper == 0) {
            error("CORBAPublishes", element.getName(), "4.1.2.3", "The CORBAEvent side cardinality can be 1 or more.");
        }
    }


    # =================================================================
    #   4.1.2.4 - Component Home
    # =================================================================
    proc checkCORBAHome (self, element)
    {
        self.checkCORBAInterface(element);
        numberCORBAManages = 0;
        for asso in self.umlOperations.associations(element) {
            if (self.umlOperations.isStereotyped(asso, "CORBAManages")) {
                numberCORBAManages = numberCORBAManages + 1;
                assoManages = asso;
                self.checkCORBAManages(assoManages);
            }
        }
        if (numberCORBAManages != 1) {
            error("CORBAHome", element.getName(), "4.1.2.4", "Exactly one CORBAManages association for each Home.");
        }
        homeSideLower = 0;
        homeSideUpper = 0;
        compSideLower = 0;
        compSideUpper = 0;
        connect = assoManages.getConnection();
        if (self.umlOperations.isStereotyped(connect.get(0).getParticipant(), "CORBAHome")
            && self.umlOperations.isStereotyped(connect.get(1).getParticipant(), "CORBAComponent")) {
            homeSideLower = connect.get(0).getMultiplicity().getRange().get(0).getLower();
            homeSideUpper = connect.get(0).getMultiplicity().getRange().get(0).getUpper();
            compSideLower = connect.get(1).getMultiplicity().getRange().get(0).getLower();
            compSideUpper = connect.get(1).getMultiplicity().getRange().get(0).getUpper();
            managedComponent = connect.get(1).getParticipant();
        } else if (self.umlOperations.isStereotyped(connect.get(1).getParticipant(), "CORBAHome")
                   && self.umlOperations.isStereotyped(connect.get(0).getParticipant(), "CORBAComponent")) {
            homeSideLower = connect.get(1).getMultiplicity().getRange().get(0).getLower();
            homeSideUpper = connect.get(1).getMultiplicity().getRange().get(0).getUpper();
            compSideLower = connect.get(0).getMultiplicity().getRange().get(0).getLower();
            compSideUpper = connect.get(0).getMultiplicity().getRange().get(0).getUpper();
            managedComponent = connect.get(0).getParticipant();
        }
        if (homeSideLower != 1 || homeSideUpper != 1) {
            error("CORBAHome", element.getName(), "4.1.2.4", "The CORBAHome side cardinality must be 1..1.");
        }
        if (compSideLower != 0 || compSideUpper == 0 || compSideUpper == 1) {
            error("CORBAHome", element.getName(), "4.1.2.4", "The CORBAComponent side cardinality must be 0..n.");
        }
        correctInheritance = true;
        numberInheritHome = 0;
        for g in element.getGeneralization().toArray() {
            if (self.umlOperations.isStereotyped(g.getParent(), "CORBAHome")) {
                numberInheritHome = numberInheritHome + 1;
                superHome = g.getParent();
                managedComponentBySuperHome = managedComponent(superHome);
                if (managedComponent != managedComponentBySuperHome
                    && !managedComponent._is_a(managedComponentBySuperHome._type)) {
                    error("CORBAHome", element.getName(), "4.1.2.4", "If CORBAHome h1 inherits from CORBAHome h2 and h2 manages CORBAComponent C2 then h1 must manage C2 or any other component C1 that inherits from C2.");
                }
                primaryKey = "notset";
                if (assoManages._is_a(org.omg.uml.core.UmlAssociationClass))
                    primaryKey = assoManages;
                primaryKeySuperHome = primaryKey(superHome, managedComponentBySuperHome);
                if (primaryKeySuperHome != "notset" && primaryKey != primaryKeySuperHome && !primaryKey._is_a(primaryKeySuperHome._type)) {
                    error("CORBAHome", element.getName(), "4.1.2.4", "If CORBAHome h1 inherits from h2, and CORBAHome h2 is associated with primary key k2 then h1 must be associated with k2 or with a primary key k1 that inherits from k2.");
                }
            } else if (self.umlOperations.isStereotyped(g.getParent(), "CORBAInterface")) {
                if (self.umlOperations.isStereotyped(g, "CORBASupports")) {
                    error("CORBAHome", element.getName(), "4.1.2.4", "Each CORBAHome inheritance from a CORBAInterface must be stereotyped CORBASupports.");
                }
            } else {
                correctInheritance = false;
            }
        }
        if (!correctInheritance || numberInheritHome > 1) {
            error("CORBAHome", element.getName(), "4.1.2.4", "A CORBAHome can inherit from one CORBAHome at most.");
        }
    }


    # =================================================================
    #   Returns the component managed by a home
    # =================================================================
    proc managedComponent (home) {
        for asso in self.umlOperations.associationsOfClass(home) {
            if (self.umlOperations.isStereotyped(asso, "CORBAManages")) {
                connect = asso.getConnection();
                if (self.umlOperations.isStereotyped(connect.get(0).getParticipant(), "CORBAHome")
                    && self.umlOperations.isStereotyped(connect.get(1).getParticipant(), "CORBAComponent")) {
                    return connect.get(1).getParticipant();
                } else if (self.umlOperations.isStereotyped(connect.get(1).getParticipant(), "CORBAHome")
                    && self.umlOperations.isStereotyped(connect.get(0).getParticipant(), "CORBAComponent")) {
                    return connect.get(0).getParticipant();
                }
            }
        }
    }


    # =================================================================
    #   Returns the primary key association class of a home
    # =================================================================
    proc primaryKey (home, component) {
        all_association_class = UMLPackage.getCore().getAssociation().refAllOfType();
        for assoCls in all_association_class {
            connect = assoCls.getConnection();
            if (connect.size() == 2) {
                if ((connect.get(0) == home && connect.get(1) == component) || (connect.get(0) == component && connect.get(1) == home))
                    return assoCls;
            }
        }
        return "notset";
    }


    # =================================================================
    #   4.1.2.4 - Component Home - CORBAManages
    # =================================================================
    proc checkCORBAManages (self, element)
    {
        homeSide = "notset";
        componentSide = "notset";
        for connect in element.getConnection().toArray() {
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAComponent")) {
                componentSide = connect;
            }
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAHome")) {
                homeSide = connect;
            }
        }
        if (homeSide == "notset" || componentSide == "notset") {
            error("CORBAManages", element.getName(), "4.1.2.4", "It's an association between a CORBAHome and a CORBAComponent.");
        }
    }


    # =================================================================
    #   4.1.2.4 - Component Home - CORBAPrimaryKey
    # =================================================================
    proc checkCORBAPrimaryKey (self, element)
    {
        numberPublicAttribute = 0;
        for feat in element.getFeature().toArray() {
            if (feat._is_a(org.omg.uml.core.UmlAttribute)) {
                if (feat.getVisibility() == org.omg.uml.data_types.VisibilityKindEnum.forName("vk_public"))
                    numberPublicAttribute = numberPublicAttribute + 1;
            }
        }
        if (numberPublicAttribute == 0)
            error("CORBAPrimaryKey", element.getName(), "4.1.2.4", "A PrimaryKey must own a public attribute.");
    }


    # =================================================================
    #   4.1.2.4 - Component Home - CORBAHomeFactory
    # =================================================================
    proc checkCORBAHomeFactory (self, element)
    {
        for param in element.getParameter().toArray() {
            if (param.getKind() != org.omg.uml.data_types.ParameterDirectionKindEnum.forName("pdk_in"))
                error("CORBAHomeFactory", element.getName(), "4.1.2.4", "A CORBAHomeFactory operation has only input parameters.");
        }
        if (!isStereotyped(element.getOwner(), "CORBAHome")) {
            error("CORBAHomeFactory", element.getName(), "4.1.2.4", "A CORBAHomeFactory can only be defined in a CORBAHome.");
        }
    }


    # =================================================================
    #   4.1.2.4 - Component Home - CORBAHomeFinder
    # =================================================================
    proc checkCORBAHomeFinder (self, element)
    {
        for param in element.getParameter() {
            if (param.getKind() != org.omg.uml.data_types.ParameterDirectionKind.forName("pdk_in"))
                error("CORBAHomeFinder", element.getName(), "4.1.2.4", "A CORBAHomeFinder has only input parameters.");
        }
        if (!isStereotyped(element.getOwner(), "CORBAHome")) {
            error("CORBAHomeFinder", element.getName(), "4.1.2.4", "A CORBAHomeFinder can only be defined in a CORBAHome.");
        }
    }


}
