/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.objectweb.openccm.browser.util.ior;

public final class InternetProfile  {
    private static final int DEFAULT_IIOP_PORT = 2089;
    public byte major, minor;
    public String host;
    public int port;
    public byte object_key[];
    public org.omg.IOP.TaggedComponent components[];

    private boolean Check() {
        if (host == null || object_key == null) {
            return false;
        }

        if (components != null) {
            for (int i = 0; i < components.length; i++) {
                if (components[i].component_data == null) {
                    return false;
                }
            }
        }

        return true;
    }

    void Read_ior(byte profile_data[]) {
        ModestInputStream in = new ModestInputStream(profile_data);
        major = in.ReadOctet();
        minor = in.ReadOctet();
        host = in.ReadString();
        port = in.ReadUShort();
        object_key = in.ReadOctetArray();

        if (major >= 1 && minor >= 1) {
            int ncomponents = in.ReadULong();
            components = new org.omg.IOP.TaggedComponent[ncomponents];

            for (int i = 0; i < ncomponents; i++) {
                org.omg.IOP.TaggedComponent component = new org.omg.IOP.TaggedComponent();
                component.tag = in.ReadULong();
                component.component_data = in.ReadOctetArray();
                components[i] = component;
            }
        }
    }

    void Read_corbaloc(String address) {
        // check protocol id
        int index;

        if (address.startsWith(":")) {
            index = 1;
        } else if (address.length() >= 5 && address.substring(0, 5).compareToIgnoreCase("iiop:") == 0) {
            index = 5;
        } else {
            throw new org.omg.CORBA.INV_OBJREF("Bad address: " + address + " (<iiop_id> not found)");
        // read version number
        }
        int ver = address.indexOf('@', index);

        if (ver == - 1) {
            major = 1;
            minor = 0;
        } else {
            int dot = address.indexOf('.');

            if (dot == - 1 || dot > ver) {
                throw new org.omg.CORBA.INV_OBJREF("Bad version number: \"" + address.substring(index, ver) + '"');
            }

            try {
                major = (byte)Integer.parseInt(address.substring(index, dot));
                minor = (byte)Integer.parseInt(address.substring(dot + 1, ver));
            } catch (NumberFormatException exc) {
                throw new org.omg.CORBA.INV_OBJREF("Bad version number: \"" + address.substring(index, ver) + '"');
            }

            index = ver + 1;
        }
        // read address
        int colon = address.indexOf(':', index);

        if (colon == - 1) {
            host = address.substring(index);
            port = DEFAULT_IIOP_PORT;
        } else {
            host = address.substring(index, colon);

            try {
                port = Integer.parseInt(address.substring(colon + 1));
            } catch (NumberFormatException exc) {
                throw new org.omg.CORBA.INV_OBJREF("Bad port number: \"" + address.substring(colon + 1) + '"');
            }
        }

        if (host.length() == 0) {
            throw new org.omg.CORBA.INV_OBJREF("No host/IP address found");
        }

        components = null;
    }

    byte[] Write_ior(boolean little_endian) {
        if (!Check()) {
            throw new org.omg.CORBA.INV_OBJREF();
        }
        ModestOutputStream out = new ModestOutputStream(little_endian);

        out.WriteOctet(major);
        out.WriteOctet(minor);
        out.WriteString(host);
        out.WriteUShort((short)port);
        out.WriteOctetArray(object_key);

        if (major >= 1 && minor >= 1) {
            if (components == null) {
                out.WriteULong(0);
            } else {
                out.WriteULong(components.length);

                for (int i = 0; i < components.length; i++) {
                    out.WriteULong(components[i].tag);
                    out.WriteOctetArray(components[i].component_data);
                }
            }
        }

        return out.GetBytes();
    }

    void Write_corbaloc(StringBuffer sb) {
        if (!Check()) {
            throw new org.omg.CORBA.INV_OBJREF();
        }

        sb.append(':');

        if (major != 1 || minor != 0) {
            sb.append(Integer.toString(major) + '.' + minor + '@');
        }

        sb.append(host);

        if (port != DEFAULT_IIOP_PORT) {
            sb.append(":" + port);
        }
    }

    public InternetProfile() {
    }

    public InternetProfile(InternetProfile profile) {

        Set(profile);
    }

    public void Set(InternetProfile profile) {
        major = profile.major;
        minor = profile.minor;
        host = profile.host;
        port = profile.port;

        if (profile.object_key != null) {
            object_key = new byte[profile.object_key.length];
            System.arraycopy(profile.object_key, 0, object_key, 0, object_key.length);
        } else {
            object_key = null;
        }

        if (profile.components != null) {
            components = new org.omg.IOP.TaggedComponent[profile.components.length];
            System.arraycopy(profile.components, 0, components, 0, components.length);
        } else {
            components = null;
        }
    }
}
