/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc
Contributor(s): _____________________________________.

====================================================================*/

package org.objectweb.openccm.services.trading;

// import generated service classes
import org.objectweb.openccm.services.trading.services.Service;

// import openccm corba utilities
import org.objectweb.openccm.services.trading.TraderTypeCodeUtils;

// import CosTrading definitions
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.PropertyMode;

/**
 * Utility class to perform various actions on the trading service.
 */
public class ServiceTypeActions {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The used service type repository. */
    org.omg.CosTradingRepos.ServiceTypeRepository repos_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    ServiceTypeActions (org.omg.CosTradingRepos.ServiceTypeRepository repos)
    {
        repos_ = repos;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Convert a type code identifier as expressed in the service type
     * DTD used in OpenCCM to a classical type code identifier, e.g.
     * <code>sequence&lt;type&gt;</code> for sequence type codes and
     * <code>unsigned type</code> for an unsigned type code.
     * 
     * @param typeCodeId The type code identifier such as used in the 
     * OpenCCM service type DTD.
     *
     * @return The classical type code identifier associated or the original 
     * one if not representing a sequence or an undigned type.
     */
    private String 
    convertSequence(String typeCodeId)
    {
        if (! typeCodeId.endsWith("Seq") && 
            ! typeCodeId.startsWith("unsigned")) return typeCodeId;
        if (typeCodeId.startsWith("unsigned_long")) return "unsigned long";
        if (typeCodeId.startsWith("unsigned_short")) return "unsigned short";
        if (typeCodeId.startsWith("String")) return "sequence<string>";
        if (typeCodeId.startsWith("Long")) return "sequence<long>";
        if (typeCodeId.startsWith("ULong")) return "sequence<unsigned long>";
        if (typeCodeId.startsWith("Short")) return "sequence<short>";
        if (typeCodeId.startsWith("UShort")) return "sequence<unsigned short>";
        if (typeCodeId.startsWith("Double")) return "sequence<double>";
        if (typeCodeId.startsWith("Float")) return "sequence<float>";
        if (typeCodeId.startsWith("Boolean")) return "sequence<boolean>";
        if (typeCodeId.startsWith("Octet")) return "sequence<octet>";
        if (typeCodeId.startsWith("Char")) return "sequence<char>";
        return typeCodeId;
    }

    /**
     * Returns the CosTrading property mode according to the given
     * string.
     *
     * @param mode_name The string representation of the property
     * mode.  this could be "normal", "mandatory", "readonly" or
     * "mandatory_readonly".
     *
     * @return The associated property mode. By default the
     * "PROP_NORMAL" mode is returned.
     */
    protected PropertyMode 
    getPropertyMode(String mode_name) {
        if (mode_name.startsWith("mandatory_readonly")) 
            return PropertyMode.PROP_MANDATORY_READONLY;
        if (mode_name.startsWith("mandatory")) 
            return PropertyMode.PROP_MANDATORY;
        if (mode_name.startsWith("readonly")) 
            return PropertyMode.PROP_READONLY;
        //if (mode_name.startsWith("normal")) 
        return PropertyMode.PROP_NORMAL;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Create a new service type described in a OpenCCM service type
     * descriptor in the OMG Trading Service.
     *
     * @param desc The Service Type information on the service type to
     * create contained in the descriptor.
     *
     * @return The service type incarnation number.
     */     
    public org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.IncarnationNumber
    create_service(Service desc)
        throws org.omg.CosTrading.IllegalServiceType,
        org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.ServiceTypeExists,
        org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.InterfaceTypeMismatch,
        org.omg.CosTrading.IllegalPropertyName,
        org.omg.CosTrading.DuplicatePropertyName,
        org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.ValueTypeRedefinition,
        org.omg.CosTrading.UnknownServiceType, 
        org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.DuplicateServiceTypeName
    {        
        // Retrieve the array of super type names
        java.util.List superTypesList = desc.getSuperTypeList();
        String[] superTypes = new String[superTypesList.size()];
        int i = 0;
        for (java.util.Iterator iter = superTypesList.iterator(); 
             iter.hasNext() ; i++)
        {
            org.objectweb.openccm.services.trading.services.SuperType st;
            st = (org.objectweb.openccm.services.trading.services.SuperType)iter.next();
            superTypes[i] = st.getName();
        }
        
        // Construct the list of org.omg.CosTrading.PropStruct
        java.util.List properties = desc.getPropertyList();
        org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.PropStruct[] props;
        props = new org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.PropStruct[properties.size()];
        i = 0;
        for (java.util.Iterator iter = properties.iterator(); 
             iter.hasNext() ; i++)
        {
            org.objectweb.openccm.services.trading.services.Property prop;
            prop = (org.objectweb.openccm.services.trading.services.Property)iter.next();
            org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.PropStruct cos_prop;
            cos_prop = new org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.PropStruct();
            cos_prop.name = prop.getName();
            cos_prop.value_type = TraderTypeCodeUtils.getTypeCode(convertSequence(prop.getType()));
            cos_prop.mode = getPropertyMode(prop.getMode());
            props[i] = cos_prop;
        }

        return repos_.add_type(desc.getName(), 
                               desc.getXmlinterface().getName(), 
                               props, superTypes);
    }
}
