/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

// Package dependencies.
import org.objectweb.util.cmdline.api.Console;
import java.io.PrintStream;

/**
 * This is the default Console implementation., i.e.
 * a wrapper class for standard output and error streams.
 *
 * This provides the following properties:
 *
 * - OutputStream: The standard output stream.
 *
 * - ErrorStream: The standard error stream.
 *
 * - MessageHeader: The header printed before messages.
 *
 * - SilentMessage: Flag to indicate if messages will be really printed or not.
 *
 * This provides the following methods:
 *
 * - message: Prints messages to the standard output stream.
 *
 * - error:   Prints messages to the standard error stream.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultConsole
  implements Console
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The standard output stream. */
    private PrintStream outputStream_;

    /** The standard error stream. */
    private PrintStream errorStream_;

    /** The message header. */
    private String messageHeader_;

    /** Boolean for indicating if messages should be displayed. */
    private boolean silentMessage_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    DefaultConsole()
    {
        this("", false);
    }

    /**
     * The constructor with the message header.
     *
     * @param header The message header.
     */
    public
    DefaultConsole(String header)
    {
        this(header, false);
    }

    /**
     * The constructor with the message header and
     * the silent message flag.
     *
     * @param header The message header.
     * @param silent The silent message flag.
     */
    public
    DefaultConsole(String header,
                   boolean silent)
    {
        this(System.out, System.err, header, silent);
    }

    /**
     * The constructor with all properties.
     *
     * @param output The standard output stream.
     * @param error The standard error stream.
     * @param header The message header.
     * @param silent The silent message flag.
     */
    public
    DefaultConsole(PrintStream output,
                   PrintStream error,
                   String header,
                   boolean silent)
    {
        // Inits internal state.
        outputStream_ = output;
        errorStream_ = error;
        messageHeader_ = header;
        silentMessage_ = silent;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.util.cmdline.api.Console.
    //
    // ==================================================================

    /**
     * Gets the standard output stream.
     *
     * @return The standard output stream.
     */
    public PrintStream
    getOutputStream()
    {
        return outputStream_;
    }

    /**
     * Sets the standard output stream.
     *
     * @param output The standard output stream.
     */
    public void
    setOutputStream(PrintStream output)
    {
        outputStream_ = output;
    }

    /**
     * Gets the standard error stream.
     *
     * @return The standard error stream.
     */
    public PrintStream
    getErrorStream()
    {
        return errorStream_;
    }

    /**
     * Sets the standard error stream.
     *
     * @param error The standard error stream.
     */
    public void
    setErrorStream(PrintStream error)
    {
        errorStream_ = error;
    }

    /**
     * Gets the message header.
     *
     * @return The message header.
     */
    public String
    getMessageHeader()
    {
        return messageHeader_;
    }

    /**
     * Sets the message header.
     *
     * @param header The message header.
     */
    public void
    setMessageHeader(String header)
    {
        messageHeader_ = header;
    }

    /**
     * Gets the silent flag status.
     *
     * @return The silent flag status.
     */
    public boolean
    getSilentMessage()
    {
        return silentMessage_;
    }

    /**
     * Sets the silent flag status.
     *
     * @param flag The silent flag status.
     */
    public void
    setSilentMessage(boolean flag)
    {
        silentMessage_ = flag;
    }

    /**
     * Prints a message on the standard output.
     *
     * @param text The message to print.
     */
    public void
    message(String text)
    {
        if(!silentMessage_)
            getOutputStream().println(messageHeader_ + ": " + text);
    }

    /**
     * Displays a message on the error output.
     *
     * @param text The message to print.
     */
    public void
    error(String text)
    {
        getErrorStream().println(messageHeader_ + ": " + text);
    }
}
