# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# --------------------------------------------------------------------
# $Id: CIDLChecker.cs,v 1.1 2004/04/15 09:09:01 carpentier Exp $
# ====================================================================
#
#  This class provides methods to check all the constraints
#  defined in the UML Profile for CORBA Components (Version 3.1).
#
# ====================================================================

import CORBA3Checker;
import ConstraintException;

class CIDLChecker (CORBA3Checker.CORBA3Checker)
{

    # =================================================================
    #   Empty constructor for the CIDLChecker.
    # =================================================================
    proc __CIDLChecker__(self)
    {
        self.__CORBA3Checker__();
    }


    # =================================================================
    #   4.2.2.1 - CORBA Component Impl
    # =================================================================
    proc checkCORBAComponentImpl (self, element)
    {
        numberAsso = 0;
        isSegmented = false;
        for asso in self.umlOperations.associations(element) {
            for connect in asso.getConnection().toArray() {
                participant = connect.getParticipant();
                if (participant != element) {
                    # [1]
                    if (self.umlOperations.isStereotyped(participant, "CORBAComponent")) {
                        numberAsso = numberAsso + 1;
                    }
                    # [2]
                    if (self.umlOperations.otherAssociationEnd(element, connect).getAggregation() == org.omg.uml.data_types.AggregationKindEnum.forName("ak_composite")) {
                        isSegmented = true;
                        if (!self.umlOperations.isStereotyped(participant, "CORBASegment")) {
                            error("CORBAComponentImpl", element.getName(), "4.2.2.1", "The only classes that are allowed to be contained by a CORBAComponentImpl are classes with the stereotype CORBASegment.");
                        }
                    }
                }
            }
        }
        if (numberAsso == 0) {
            error("CORBAComponentImpl", element.getName(), "4.2.2.1", "The is an association between CORBAComponentImpl and CORBAComponent.");
        }
        # [3]
        if (isSegmented) {
            category = "session";
            for tv in element.getTaggedValue().toArray() {
                if (tv.getType().getTagType() == "category") {
                    category = tv.getDataValue().get(0).toLowerCase();
                }
            }
            if (category != "entity" && category != "process") {
                error("CORBAComponentImpl", element.getName(), "4.2.2.2", "The life cycle category of the CORBAComponentImpl must be 'entity' or 'process' if the component implementation is segmented.");
            }
        }
    }


    # =================================================================
    #   4.2.2.1 - CORBA Implements
    # =================================================================
    proc checkCORBAImplements (self, element)
    {
        for connect in element.getConnection().toArray() {
            errorComponent = false;
            lower = 0;
            upper = 0;
            for range in connect.getMultiplicity().getRange().toArray() {
                lower = range.getLower();
                upper = range.getUpper();
            }
            # [1]
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAComponentImpl")) {
                if (!(lower == 1 && upper != 0)) errorComponent = true;
            }
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAComponent")) {
                if (!(lower == 1 && upper == 1)) errorComponent = true;
            }
            if (errorComponent) {
                error("CORBAImplements", connect.getParticipant().getName(), "4.2.2.1", "A CORBAComponentImpl always has exactly one CORBAComponent associated while each CORBAComponent might be implemented by different types of CORBAComponentImpl.");
            }
            # [2]
            if (self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAHomeImpl")
                || self.umlOperations.isStereotyped(connect.getParticipant(), "CORBAHome")) {
                if (!(lower == 1 && upper == 1)) {
                    error("CORBAImplements", connect.getParticipant().getName(), "4.2.2.1", "Each CORBAHomeImpl in a model implements exactly one CORBAHome.");
                }
            }
        }
    }


    # =================================================================
    #   4.2.2.2 - CORBA Home Impl
    # =================================================================
    proc checkCORBAHomeImpl (self, element)
    {        
    }


    # =================================================================
    #   4.2.2.4 - CORBA Segment
    # =================================================================
    proc checkCORBASegment (self, element)
    {
        containerOk = false;
        for asso in self.umlOperations.oppositeAssociationEnds(element) {
            if (asso.getAggregation() == org.omg.uml.data_types.AggregationKindEnum.forName("ak_composite")
                && self.umlOperations.isStereotyped(asso.getParticipant(), "CORBAComponentImpl")) {
                containerOk = true;
            }
        }
        if (!containerOk) {
            error("CORBASegment", element.getName(), "4.2.2.4", "CORBASegment classes are always contained in CORBAComponentImpl.");
        }
    }


    # =================================================================
    #   4.2.2.4 - CORBA Artifact
    # =================================================================
    proc checkCORBAArtifact (self, element)
    {
        for asso in self.umlOperations.oppositeAssociationEnds(element) {
            if (asso.getAggregation() == org.omg.uml.data_types.AggregationKindEnum.forName("ak_composite")) {
                error("CORBAArtifact", element.getName(), "4.2.2.4", "The only allowed Container for ArtifactDef is ModuleDef.");
            }
        }
    }


}
