# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# --------------------------------------------------------------------
# $Id: CIDLParser.cs,v 1.1 2004/04/15 09:09:01 carpentier Exp $
# ====================================================================
#
#  This class feeds the AST from the differents CIDL elements in the
#  UML repository. This file is build according to the UML Profile
#  Profile for CORBA Components (Version 3.1) [1].
#
# ====================================================================


import NotCreatedException;
import UMLOperations;

class CIDLParser
{

    # =================================================================
    # Empty Constructor for the CIDL parser.
    # =================================================================
    proc __CIDLParser__(self, types)
    {
        self.umlOperations = UMLOperations.UMLOperations();
        self._types = types;

        JAVA.declare("org.objectweb.corba.ast.api");
        JAVA.declare("org.objectweb.corba.ast.lib");
        JAVA.declare("org.objectweb.ccm");
        JAVA.declare("java.util");
        ir = CORBA.ORB.resolve_initial_references("InterfaceRepository");
        self.ast = org.objectweb.corba.ast.lib.Repository(org.objectweb.ccm.ComponentRepositoryHelper.narrow(ir));

        self.scope = self.ast.startFileScope("");
        self.scope.create();
        self.prefix = "";
    }


    # =================================================================
    # Puts the type in the hashtable.
    # element - The UML element corresponding to the type
    # type - The type itself
    # =================================================================
    proc putType (self, element, type)
    {
        self._types[element] = type;
    }


    # =================================================================
    # Gets the type from the hashtable. If the type is not present in
    # the hashtable, the method throws the NotCreatedException.
    # element - The UML element corresponding to the type
    # return - The type itself
    # =================================================================
    proc getType (self, element)
    {
        if (!self.typeExist(element)) {
            throw NotCreatedException.NotCreatedException(element.name());
        }
        return self._types[element];
    }


    # =================================================================
    # Tests if the type is present in the hashtable.
    # element - The UML element corresponding to the type
    # return - true if the type is present, else false
    # =================================================================
    proc typeExist (self, element)
    {
        return self._types.containsKey(element);
    }


    # =================================================================
    # Feeds the Interface Repository from the UML model.
    # umlPackage - The UML Package
    # =================================================================
    proc parseModel(self, umlPackage)
    {
        self.umlPackage = umlPackage;
        self.undefinedType = false;
        try {
            self.umlOperations.setUMLPackage(umlPackage);
            packages = umlPackage.model_management_ref.package_ref.all_of_type_package;
            self.parse(packages, self.scope);
        } catch (exception) {
            throw exception;
        }
    }


    # =================================================================
    # Create some elements of the model into the Interface Repository.
    # modelElements - The elements
    # currentScope - The current scope in which create the element
    # =================================================================
    proc parse (self, modelElements, currentScope)
    {
        for element in modelElements {
            self.parseElement(element, currentScope);
        }
    }


    # =================================================================
    # Create one element of the model into the Interface Repository.
    # element - The element
    # currentScope - The current scope in which create the element
    # =================================================================
    proc parseElement (self, element, currentScope)
    {
        stereotypes = element.stereotype();
        if (stereotypes.length != 0) {
            try {
                for stereotype in stereotypes {
                    eval("self.parse" + stereotype.name() + "(element, currentScope);");
                }
            } catch (NotFound nf) {
                # Nothing
            }
        }
    }


    # =================================================================
    #   [1] - 3.5.4 - Module
    # =================================================================
    proc parseCORBAModule (self, element, currentScope)
    {
        oldPrefix = self.prefix;
        module = currentScope.declareCidlModule(element.name());
        println("declareCidlModule(", element.name(), ")");
        self.currentModule = module;
        for tv in element.tagged_value() {
            if (tv.type().tag_type() == "typePrefix" || tv.type().tag_type() == "CORBATypePrefix") {
                self.prefix = tv.data_value()[0];
            }
        }
        module.setPrefix(self.prefix);
        module.create();
        self.parse(element.owned_element(), module);
        self.prefix = oldPrefix;
        module.create();

        ## Generation of the CIDL file corresponding to this module
        println("Generation ...");
        JAVA.declare("org.objectweb.corba.generator.cidl.lib");
        JAVA.declare("org.objectweb.corba.generator.common.lib");
        gen = org.objectweb.corba.generator.cidl.lib.CIDLGenerator(self.ast);
        gen.initialize("test.cidl", "XMItoCIDL");
        gen.generate(module, org.objectweb.corba.ast.api.DeclarationKind.dk_cidl);
    }


    # =================================================================
    #   [1] - 4.2.2.1 - CORBA Component Impl
    # =================================================================
    proc parseCORBAComponentImpl (self, element, currentScope)
    {
        impl = self.getImplements(element);
        # Gets the Category Kind of the Composition
        category = "session";
        for tv in element.tagged_value() {
            if (tv.type().tag_type() == "category") {
                category = tv.data_value()[0].toLowerCase();
            }
        }
        if (category == "entity") {
            categoryKind = org.objectweb.corba.ast.api.CategoryKind.ck_entity;
        } else if (category == "process") {
            categoryKind = org.objectweb.corba.ast.api.CategoryKind.ck_process;
        } else if (category == "service") {
            categoryKind = org.objectweb.corba.ast.api.CategoryKind.ck_service;
        } else {
            category = "session";
            categoryKind = org.objectweb.corba.ast.api.CategoryKind.ck_session;
        }
        # Composition creation
        category = category.substring(0,1).toUpperCase() + category.substring(1).toLowerCase();
        comp = currentScope.startComposition(impl.name() + category + "Composition");
        comp.setCategoryKind(categoryKind);
        comp.create();
        # home executor
        homeExecutor = comp.startHomeExecutor("HomeImpl");
        homeExecutor.create();
        # implements
        homeExecutor.setHomeType(self.getType(self.getHome(impl)));
        # manages
        exec = homeExecutor.startExecutor("ComponentImpl");
        # segment
        for connect in self.umlOperations.oppositeAssociationEnds(element) {
            segmentElement = connect.participant();
            if (self.umlOperations.isStereotyped(segmentElement, "CORBASegment")) {
                self.createCORBASegment(segmentElement, exec);
            }
        }
        exec.create();
        homeExecutor.create();
        comp.create();
    }

    proc getImplements (self, element)
    {
        for asso in self.umlOperations.associations(element) {
            if (self.umlOperations.isStereotyped(asso, "CORBAImplements")) {
                for connect in asso.connection() {
                    if (connect.participant() != element) {
                        return connect.participant();
                    }
                }
            }
        }
        return JAVA.null;
    }

    proc getHome(self, element)
    {
        for asso in self.umlOperations.associations(element) {
            if (self.umlOperations.isStereotyped(asso, "CORBAManages")) {
                for connect in asso.connection() {
                    if (connect.participant() != element) {
                        return connect.participant();
                    }
                }
            }
        }
        return JAVA.null;
    }


    # =================================================================
    #   [1] - 4.2.2.1 - CORBA Implements
    # =================================================================
    proc parseCORBAImplements (self, element, currentScope)
    {
        println("CORBAImplements");
    }


    # =================================================================
    #   [1] - 4.2.2.1 - CORBA Manages
    # =================================================================
    proc parseCORBAManages (self, element, currentScope)
    {
        println("CORBAManages");
    }


    # =================================================================
    #   [1] - 4.2.2.2 - CORBA Home Impl
    # =================================================================
    proc parseCORBAHomeImpl (self, element, currentScope)
    {
        println("CORBAHomeImpl");
    }


    # =================================================================
    #   [1] - 4.2.2.4 - CORBA Segment
    # =================================================================
    proc createCORBASegment (self, element, currentScope)
    {
        println("CORBASegment");
        segment = currentScope.startSegment(element.name());
        # provides facet
        for artifactEnds in self.umlOperations.oppositeAssociationEnds(element) {
            artifact = artifactEnds.participant();
            if (self.umlOperations.isStereotyped(artifact, "CORBAArtifact")) {
                segment.getFacetList().add(artifact.name());
            }
        }
        # provides sink
        for tv in element.tagged_value() {
            if (tv.type().tag_type() == "features") {
                featuresList = tv.data_value()[0];
                featuresToken = java.util.StringTokenizer(featuresList, "[, ]");
                while (featuresToken.hasNext()) {
                    featuresToken.nextToken();
                    feature_name = featuresToken.substring(0, featuresToken.index(":"));
                    storage_member_name = featuresToken.substring(featuresToken.index(":")+1);
                    currentScope.getFeatureDelegationList().add(feature_name, storage_member_name);
                }
            }
        }
        segment.create();
    }


    # =================================================================
    #   [1] - 4.2.2.4 - CORBA Artifact
    # =================================================================
    proc parseCORBAArtifact (self, element, currentScope)
    {
        println("CORBAArtifact");
    }



}
