/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Frdric Briclet, Romain Rouvoy.

--------------------------------------------------------------------
$Id: Launcher.java,v 1.1 2004/02/05 20:29:57 rouvoy Exp $
====================================================================*/
package org.objectweb.util.launcher;


import org.objectweb.util.launcher.option.OptionContext;
import org.objectweb.util.launcher.option.OptionRun ;
import org.objectweb.util.trace.TraceSystem;

/**
 * Launcher is used to run classes (call the "main" method).
 * Properties are read from an xml file.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * @version 0.1
 */
public class Launcher 
{   
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /** list of run to execute */
    protected String runid_ ;
    /** list of context to execute */
    protected String opt_ ;
    /** file containing the descriptions */
    protected String file_ ;
    /** list of arguments */
    protected String args_ ;
 
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    
    /**
     * Default constructor.
     */
    public Launcher() {
        this.runid_ = "" ;
        this.opt_ = "" ;
        this.file_ = "" ;
        this.args_ = "" ;
    }

    // ==================================================================
    //
    // Internal class.
    //
    // ==================================================================
    /**
     * Internal class in charge to launch an new JVM for new program
     * to launch    
     */
    protected class ProcessStarter  
            extends Thread 
    {
        /* String command to execute */
        protected String toExecute ;
        /* Private handler on the process launched */
        protected Process p=null;
        
        /**
         *  Accessor method to retrieve the launched process
         **/
        public Process getProcess() { 
            return p; 
        }

        /**
         * Main constructor 
         * @param toLaunch String line to execute
         */
        public ProcessStarter(String toLaunch ) {
            toExecute = System.getProperty("java.home") +
                "/bin/java -classpath " +
                System.getProperty("java.class.path") + 
                " org.objectweb.util.launcher.Launcher " +
                toLaunch;
        }

        /**
         * Start method of the thread.
         */
         public void run() {
             try { // launch the process          
                 p=Runtime.getRuntime().exec(toExecute);          
             } catch(Exception ex) {		
                 TraceSystem.get("launcher").error("Exception raised: "+ex);
                 throw new LauncherException(ex);
             }
         }
    }


    /**
     * Defines the run to execute.
     * @param run identifier of the run.
     */
    public void setRunid(String run){ 
        runid_ += OptionRun.create(run); 
    }

    /**
     * Specify a particular context to use.
     * @param opt identifier of the context.
     */
    public void addContext(String opt) { 
        opt_ += OptionContext.create(opt); 
    }

    /**
     * Specify the configuration file to use.
     * @param file the URL of the file.
     */
    public void setFile(String file){ 
        file_ = file ; 
    }

    /**
     * Defines additional arguments for the application.
     * @param args list of arguments
     */
    public void addArgs(String args) { 
        args_ += " " + args ; 
    }

    /**
     * Provides the command line to use. 
     * @return the full command line.
     */
    private String getCommandLine() {
        return runid_ + " " + opt_ + " " + " " + file_ ;
    }
    
    /**
     * Starts the application.
     * @return the process representing the launched application.
     */
    public Process start() {        
        try {
            ProcessStarter ps = new ProcessStarter(getCommandLine());
            // start the thread
            ps.start();
            // wait for application starting
            while(ps.getProcess()==null)
                Thread.sleep(1000);
            return ps.getProcess();
        } catch(Exception ex) {
            TraceSystem.get("launcher").error("Exception raised: " + ex);
            throw new LauncherException(ex);
        }
    }

    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
  
    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args) {
        LauncherApplication application = new LauncherApplication();
        application.runMain(args);
    }
}
