/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

--------------------------------------------------------------------
$Id: ContextDescription.java,v 1.3 2004/02/13 18:14:48 rouvoy Exp $
====================================================================*/

package org.objectweb.util.launcher.parser ;


import java.util.Properties ;

import org.objectweb.util.launcher.StringList ;
import org.objectweb.util.trace.TraceSystem;


/**
 * Representation of an context.<BR>
 * <p>
 * An context describe additionnal informations on an application.
 * It contains:
 * <li>
 *   <ul>an identifier</ul>
 *   <ul>a list of arguments</ul>
 *   <ul>a list of properties</ul>
 *   <ul>a list of Java classes</ul>
 * </li>
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class ContextDescription
{
    /** The name of the context. */
    protected String name_ ;

    /** The arguments list. */
    protected StringList arguments_;

    /** The properties list. */
    protected Properties properties_;

    /** The class list. */
    protected StringList classpath_ ;
    
    /** The description list. */
    protected StringList descriptions_;

    /**
     * Default Constructor.
     */
    public ContextDescription() {
        this.arguments_ = new StringList();
        this.classpath_ = new StringList();
        this.properties_ = new Properties();
        this.descriptions_ = new StringList();
    }


    /**
     * Defines the identifer of the context.
     *
     * @param name identifier of the context.
     */
    public void setName(String name) { 
        TraceSystem.get("description").debug("Setting name: \""+name+"\"");
        name_ = name; 
    }

    /**
     * Provides the identifier of the context.
     *
     * @return identifier of the context.
     */
    public String getName() { 
        return name_; 
    }


    /**
     * Adds an argument in the content of the context.
     *
     * @param arg argument to add.
     */
    public void addArgument(String arg) { 
        TraceSystem.get("description").debug("Adding argument: \""+arg+"\"");
        arguments_.add(arg); 
    }
    
    /**
     * Provides the list of arguments for the context.
     *
     * @return the list of arguments.
     */
    public StringList getArguments() { 
        return arguments_; 
    }

    
    /**
     * Adds a property in the content of the context.
     *
     * @param key the key of the property.
     * @param value the value of the property.
     */
    public void addProperty(String key, String value) { 
        TraceSystem.get("description").debug("Adding property: \""+key+"\"=\""+value+"\"");
        properties_.put(key,value); 
    }
    
    /**
     * Provides the list of properties for the context.
     *
     * @return the list of Java Properties.
     */
    public Properties getProperties() { 
        return properties_; 
    }

    /**
     * Adds an element in the classpath of the context.
     *
     * @param classname the name of the class to add.
     */
    public void addClass(String classname) { 
        TraceSystem.get("description").debug("Adding class: \""+classname+"\"");
        classpath_.add(classname); 
    }

    /**
     * Provides the list of classes considered by the context.
     *
     * @return the list of classes to load.
     */
    public StringList getClasses() { 
        return classpath_; 
    }
    
    /**
     * Adds the identifier of a loaded description.
     * @param desc the description to add.
     */
    public void addDescription(String desc) {
        if (checkDescription(desc))
            TraceSystem.get("description").debug("Description"+desc+" has still been loaded!");
        else
            descriptions_.add(desc);
    }
    
    /**
     * Checks if the description have still been loaded for this context (true if loaded).
     * @param desc the description to check.
     * @return true if the description desc have still been added.
     */
    public boolean checkDescription(String desc) {
        return descriptions_.contains(desc);
    }
}
